#!/usr/bin/env python3

#   Gimp-Python - allows the writing of Gimp plugins in Python.
#   Copyright (C) 1997  James Henstridge <james@daa.com.au>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
gi.require_version('Gimp', '3.0')
gi.require_version('GimpUi', '3.0')
from gi.repository import Gimp
from gi.repository import GimpUi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from gi.repository import GObject
from gi.repository import Gio
from gi.repository import GLib

import sys
import warnings
import pyconsole
#import gimpshelf, gimpui, pyconsole

import gettext
textdomain = "gimp30-python"
gettext.textdomain(textdomain)
_ = gettext.gettext

PROC_NAME = 'python-fu-console'

RESPONSE_BROWSE, RESPONSE_CLEAR, RESPONSE_SAVE = range(3)

def run(procedure, config, data):
    GimpUi.init ("python-console.py")

    namespace = {'__builtins__': __builtins__,
                 '__name__': '__main__', '__doc__': None,
                 'Babl': gi.repository.Babl,
                 'cairo': gi.repository.cairo,
                 'Gdk': gi.repository.Gdk,
                 'Gegl': gi.repository.Gegl,
                 'Gimp': gi.repository.Gimp,
                 'GimpUi': gi.repository.GimpUi,
                 'Gio': gi.repository.Gio,
                 'Gtk': gi.repository.Gtk,
                 'GdkPixbuf': gi.repository.GdkPixbuf,
                 'GLib': gi.repository.GLib,
                 'GObject': gi.repository.GObject,
                 'Pango': gi.repository.Pango }

    class GimpConsole(pyconsole.Console):
        def __init__(self, quit_func=None, initial_history=None):
            banner = ('GIMP %s Python Console\nPython %s\n' %
                      (Gimp.version(), sys.version))
            pyconsole.Console.__init__(self,
                                       locals=namespace, banner=banner,
                                       quit_func=quit_func,
                                       initial_history=initial_history)
        def _commit(self):
            pyconsole.Console._commit(self)
            Gimp.displays_flush()

    class ConsoleDialog(GimpUi.Dialog):
        def __init__(self):
            use_header_bar = Gtk.Settings.get_default().get_property("gtk-dialogs-use-header")
            GimpUi.Dialog.__init__(self, use_header_bar=use_header_bar)
            self.set_property("help-id", PROC_NAME)
            Gtk.Window.set_title(self, _("Python Console"))
            Gtk.Window.set_role(self, PROC_NAME)
            Gtk.Dialog.add_button(self, _("_Save"), Gtk.ResponseType.OK)
            Gtk.Dialog.add_button(self, _("Cl_ear"), RESPONSE_CLEAR)
            Gtk.Dialog.add_button(self, _("_Browse..."), RESPONSE_BROWSE)
            Gtk.Dialog.add_button(self, _("_Close"), Gtk.ResponseType.CLOSE)

            Gtk.Widget.set_name (self, PROC_NAME)
            GimpUi.Dialog.set_alternative_button_order_from_array(self,
                                                                [ Gtk.ResponseType.CLOSE,
                                                                  RESPONSE_BROWSE,
                                                                  RESPONSE_CLEAR,
                                                                  Gtk.ResponseType.OK ])

            history = config.get_property('history')
            self.cons = GimpConsole(quit_func=lambda: Gtk.main_quit(), initial_history=history)

            self.style_set (None, None)

            self.connect('response', self.response)
            self.connect('style-set', self.style_set)

            self.browse_dlg = None
            self.save_dlg = None

            vbox = Gtk.VBox(homogeneous=False, spacing=12)
            vbox.set_border_width(12)
            contents_area = Gtk.Dialog.get_content_area(self)
            contents_area.pack_start(vbox, True, True, 0)

            scrl_win = Gtk.ScrolledWindow()
            scrl_win.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.ALWAYS)
            vbox.pack_start(scrl_win, True, True, 0)

            scrl_win.add(self.cons)

            width, height = self.cons.get_default_size()
            minreq, requisition = Gtk.Widget.get_preferred_size(scrl_win.get_vscrollbar())
            sb_width  = requisition.width
            sb_height = requisition.height

            # Account for scrollbar width and border width to ensure
            # the text view gets a width of 80 characters. We don't care
            # so much whether the height will be exactly 40 characters.
            Gtk.Window.set_default_size(self, width + sb_width + 2 * 12, height)

        def style_set(self, old_style, user_data):
            pass
            #style = Gtk.Widget.get_style (self)
            #self.cons.stdout_tag.set_property ("foreground", style.text[Gtk.StateType.NORMAL])
            #self.cons.stderr_tag.set_property ("foreground", style.text[Gtk.StateType.INSENSITIVE])

        def response(self, dialog, response_id):
            if response_id == RESPONSE_BROWSE:
                self.browse()
            elif response_id == RESPONSE_CLEAR:
                self.cons.banner = None
                self.cons.clear()
            elif response_id == Gtk.ResponseType.OK:
                self.save_dialog()
            else:
                # Store up to 100 commands.
                config.set_property('history', self.cons.history.items[-100:])
                Gtk.main_quit()

            self.cons.grab_focus()

        def command_for_procedure(self, proc_name):
            '''
            Assemble string of Python code that when eval'd
            will call proc_name with contrived arguments.

            The purpose is to generate a template for a call
            to the PDB procedure the user has selected.
            The call MIGHT work as is.
            Otherwise, the names of the arguments might be enough
            that the user can figure out how to edit the template.

            The code will run in the environment of the console/browser,
            which is not the GIMP v2 GimpFu environment
            but the GIMP v3 PyGObject introspected environment.

            If ever GimpFu module is resurrected, and Python console imports it,
            then revert this code to its v2 form.
            '''
            proc = Gimp.get_pdb().lookup_procedure(proc_name)
            if proc is None:
                return None

            return_values = proc.get_return_values()
            param_specs   = proc.get_arguments()

            cmd  = f"procedure = Gimp.get_pdb().lookup_procedure('{proc_name}'); "
            cmd += f"config = procedure.create_config(); "

            for arg in param_specs:
              if arg.name == 'run-mode':
                # Special handling for run mode.
                cmd += "config.set_property('" + arg.name + "', Gimp.RunMode.INTERACTIVE); "
              elif type(arg) == Gimp.ParamCoreObjectArray:
                # Special handling for GimpCoreObjectArray parameters
                cmd += "config.set_core_object_array('" + arg.name + "', " + arg.name.replace('-', '_') + "); "
              else:
                cmd += "config.set_property('" + arg.name + "', " + arg.name.replace('-', '_') + "); "

            cmd += f"result = procedure.run(config); "

            cmd += f"success = result.index(0)"
            if len(return_values) > 0:
              i = 1
              for retval in return_values:
                cmd += '; {} = result.index({})'.format(retval.name.replace('-', '_'), i)
                i += 1

            return cmd

        def browse_response(self, dlg, response_id):
            if response_id != Gtk.ResponseType.APPLY:
                Gtk.Widget.hide(dlg)
                return

            proc_name = dlg.get_selected()

            if not proc_name:
                # Apply button was enabled without a selection?
                return

            cmd = self.command_for_procedure(proc_name)
            if cmd is None:
                # Should not happen.  We browsed a name not in the PDB?
                return

            buffer = self.cons.buffer

            lines = buffer.get_line_count()
            iter = buffer.get_iter_at_line_offset(lines - 1, 4)
            buffer.delete(iter, buffer.get_end_iter())
            buffer.place_cursor(buffer.get_end_iter())
            buffer.insert_at_cursor(cmd)
            # not insert a newline, user can edit and then "enter" the command

        def browse(self):
            if not self.browse_dlg:
                use_header_bar = Gtk.Settings.get_default().get_property("gtk-dialogs-use-header")
                dlg = GimpUi.ProcBrowserDialog(use_header_bar=use_header_bar)
                Gtk.Window.set_title(dlg, _("Python Procedure Browser"))
                Gtk.Window.set_role(dlg, PROC_NAME)
                Gtk.Dialog.add_button(dlg, _("_Apply"), Gtk.ResponseType.APPLY)
                Gtk.Dialog.add_button(dlg, _("_Close"), Gtk.ResponseType.CLOSE)

                Gtk.Dialog.set_default_response(self, Gtk.ResponseType.OK)
                GimpUi.Dialog.set_alternative_button_order_from_array(dlg,
                                                                    [ Gtk.ResponseType.CLOSE,
                                                                      Gtk.ResponseType.APPLY ])

                dlg.connect('response', self.browse_response)
                dlg.connect('row-activated',
                            lambda dlg: dlg.response(Gtk.ResponseType.APPLY))

                self.browse_dlg = dlg

            Gtk.Window.present(self.browse_dlg)

        def save_response(self, dlg, response_id):
            if response_id == Gtk.ResponseType.DELETE_EVENT:
                self.save_dlg = None
                return
            elif response_id == Gtk.ResponseType.OK:
                filename = dlg.get_filename()

                try:
                    logfile = open(filename, 'w')
                except IOError as e:
                    Gimp.message(_("Could not open '%s' for writing: %s") %
                                 (filename, e.strerror))
                    return

                buffer = self.cons.buffer

                start = buffer.get_start_iter()
                end = buffer.get_end_iter()

                log = buffer.get_text(start, end, False)

                try:
                    logfile.write(log)
                    logfile.close()
                except IOError as e:
                    Gimp.message(_("Could not write to '%s': %s") %
                                 (filename, e.strerror))
                    return

            Gtk.Widget.hide(dlg)

        def save_dialog(self):
            if not self.save_dlg:
                dlg = Gtk.FileChooserDialog(action=Gtk.FileChooserAction.SAVE)
                Gtk.Window.set_title(dlg, _("Save Python-Fu Console Output"))
                Gtk.Window.set_transient_for(dlg, self)
                Gtk.Dialog.add_button(dlg, _("_Cancel"), Gtk.ResponseType.CANCEL)
                Gtk.Dialog.add_button(dlg, _("_Save"), Gtk.ResponseType.OK)
                Gtk.Dialog.set_default_response(self, Gtk.ResponseType.OK)

                with warnings.catch_warnings():
                    warnings.filterwarnings('ignore')
                    Gtk.FileChooserDialog.set_alternative_button_order_from_array(dlg,
                                                                                [ Gtk.ResponseType.OK,
                                                                                  Gtk.ResponseType.CANCEL ])

                dlg.connect('response', self.save_response)

                self.save_dlg = dlg

            self.save_dlg.present()

        def run(self):
            Gtk.Widget.show_all(self)
            Gtk.main()

    ConsoleDialog().run()

    return procedure.new_return_values(Gimp.PDBStatusType.SUCCESS, GLib.Error())

class PythonConsole (Gimp.PlugIn):
    ## GimpPlugIn virtual methods ##
    def do_set_i18n(self, name):
        gettext.bindtextdomain(textdomain, Gimp.locale_directory())
        return True, 'gimp30-python', None

    def do_query_procedures(self):
        return [ PROC_NAME ]

    def do_create_procedure(self, name):
        if name == PROC_NAME:
            procedure = Gimp.Procedure.new(self, name,
                                           Gimp.PDBProcType.PLUGIN,
                                           run, None)
            procedure.set_menu_label(_("Python _Console"))
            procedure.set_documentation(_("Interactive GIMP Python interpreter"),
                                        _("Type in commands and see results"),
                                        "")
            procedure.set_attribution("James Henstridge",
                                      "James Henstridge",
                                      "1997-1999")
            procedure.add_enum_argument ("run-mode", _("Run mode"),
                                         _("The run mode"), Gimp.RunMode,
                                         Gimp.RunMode.INTERACTIVE,
                                         GObject.ParamFlags.READWRITE)
            procedure.add_string_array_aux_argument ("history",
                                                     "Command history", "Command history",
                                                     GObject.ParamFlags.READWRITE)
            procedure.add_menu_path ("<Image>/Filters/Development/Python-Fu")

            return procedure
        return None

Gimp.main(PythonConsole.__gtype__, sys.argv)



#!/usr/bin/env python3

NEW_IMAGE_WIDTH=1920
NEW_IMAGE_HEIGHT=2001

image = Gimp.Image.new(NEW_IMAGE_WIDTH, NEW_IMAGE_HEIGHT, Gimp.ImageBaseType.RGB)
text_layer = Gimp.TextLayer.new(image, "hello world", Gimp.context_get_font(), 20, Gimp.Unit.point())
image.insert_layer(text_layer, None, 0)
text_layer = Gimp.TextLayer.new(image, "annyeong uju", Gimp.context_get_font(), 20, Gimp.Unit.point())
image.insert_layer(text_layer, None, 0)

images = Gimp.get_images()
layers = image.get_layers()
gimp_assert('Verify start state', len(images) == 1 and images[0] == image and len(layers) == 2)

options = Gimp.ExportOptions()
options.set_property ("capabilities",
                      Gimp.ExportCapabilities.CAN_HANDLE_RGB |
                      Gimp.ExportCapabilities.CAN_HANDLE_ALPHA)

delete, export_image = options.get_image(image)

gimp_assert('Gimp.ExportOptions.get_image() created a new image', delete == Gimp.ExportReturn.EXPORT and export_image != image)
export_layers = export_image.get_layers()
gimp_assert('The new image has a single layer', len(export_layers) == 1)

merged_layer = image.merge_visible_layers(Gimp.MergeType.CLIP_TO_IMAGE)
merged_layer.resize_to_image_size()

buffer1 = merged_layer.get_buffer()
buffer2 = export_layers[0].get_buffer()
l1 = buffer1.get(buffer1.get_extent(), 1.0, None, Gegl.AbyssPolicy.NONE)
l2 = buffer2.get(buffer2.get_extent(), 1.0, None, Gegl.AbyssPolicy.NONE)
gimp_assert("Compare export buffer with original image's merged buffer", l1 == l2)



#!/usr/bin/env python3

"""
performance-log-resolve.py -- Resolve GIMP performance log backtrace symbols
Copyright (C) 2018  Ell

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Usage: performance-log-resolve.py < infile > outfile
"""

from xml.etree import ElementTree
import sys

empty_element = ElementTree.Element ("")

# Read performance log from STDIN
log = ElementTree.fromstring (sys.stdin.buffer.read ())

address_map = log.find ("address-map")

if address_map:
    # Create address dictionary
    addresses = {}

    for address in address_map.iterfind ("address"):
        addresses[address.get ("value")] = list (address)

    # Resolve addresses in backtraces
    for sample in (log.find ("samples") or empty_element).iterfind ("sample"):
        for thread in sample.find ("backtrace") or ():
            for frame in thread:
                address = addresses.get (frame.get ("address"))

                if address:
                    frame.text = "\n"
                    frame.extend (address)

    # Remove address map
    log.remove (address_map)

# Write performance log to STDOUT
sys.stdout.buffer.write (ElementTree.tostring (log))



#!/usr/bin/env python3

"""
generate-welcome-dialog-data.py -- Generate app/dialogs/welcome-dialog-data.h
Copyright (C) 2022 Jehan

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Usage: generate-welcome-dialog-data.py
"""

import argparse
import os.path
import re
import sys
import xml.etree.ElementTree as ET

tools_dir   = os.path.dirname(os.path.realpath(__file__))
desktop_dir = os.path.join(tools_dir, '../desktop')
outdir      = os.path.join(tools_dir, '../app/dialogs')

infile      = os.path.join(desktop_dir, 'org.gimp.GIMP.appdata.xml.in.in')
outfile     = os.path.join(outdir, 'welcome-dialog-data.h')

def parse_appdata(infile, version):
  introduction  = []
  release_texts = []
  release_demos = []

  version = version.lower()
  spaces = re.compile(r'\s+')
  tree = ET.parse(infile)
  root = tree.getroot()
  releases_node = root.find('releases')
  releases = releases_node.findall('release')
  for release in releases:
    if 'version' in release.attrib and \
       (release.attrib['version'].lower() == version or
        release.attrib['version'].replace('~', '-').lower() == version):
      intro = release.findall('./description/p')
      for p in intro:
        # Naive conversion for C strings, but it will probably fit for
        # most cases.
        p = p.text.strip()
        p = p.replace('\\', '\\\\')
        p = p.replace('"', '\\"')
        # All redundant spaces unwanted as XML merges them anyway.
        introduction += [spaces.sub(' ', p)]

      items = release.findall('./description/ul/li')
      for item in items:
        text = item.text.strip()
        text = text.replace('\\', '\\\\')
        text = text.replace('"', '\\"')
        demo = None
        if 'demo' in item.attrib:
          demo = item.attrib['demo']
          # All spaces unneeded in demo string.
          demo = demo.replace(' ', '')
        release_texts += [spaces.sub(' ', text)]
        release_demos += [demo]
      break

  return introduction, release_texts, release_demos

if __name__ == "__main__":
  parser = argparse.ArgumentParser()
  parser.add_argument('version')
  parser.add_argument('--header', action='store_true')
  args = parser.parse_args(sys.argv[1:])

  top_comment = '''/* GIMP - The GNU Image Manipulation Program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * welcome-dialog-data.h
 * Copyright (C) 2022 Jehan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************
 * This file is autogenerated by tools/generate-welcome-dialog-data.py *
 ***********************************************************************
 *
 * Modify the python script or desktop/org.gimp.GIMP.appdata.xml.in.in
 * instead of this one
 * Then run tools/generate-welcome-dialog-data.py again.
 */

'''
  print(top_comment)

  intro_p, items, demos = parse_appdata(infile, args.version)

  if args.header:
    print('#pragma once\n\n')

    print('extern gint          gimp_welcome_dialog_n_items;')
    print('extern const gchar * gimp_welcome_dialog_items[];')
    print('extern const gchar * gimp_welcome_dialog_demos[];')
    print()
    print('extern gint          gimp_welcome_dialog_intro_n_paragraphs;')
    print('extern const gchar * gimp_welcome_dialog_intro[];')

  else:
    print('#include "config.h"')
    print('#include <glib.h>')
    print()

    print('const gint   gimp_welcome_dialog_n_items = {};'.format(len(demos)))
    print()
    print('const gchar *gimp_welcome_dialog_items[] =')
    print('{')
    for item in items:
      print('  "{}",'.format(item))
    print('  NULL,\n};')
    print()
    print('const gchar *gimp_welcome_dialog_demos[] =')
    print('{')
    for demo in demos:
      if demo is None:
        print('  NULL,')
      else:
        print('  "{}",'.format(demo))
    print('  NULL,\n};')
    print()
    print('const gint   gimp_welcome_dialog_intro_n_paragraphs = {};'.format(len(intro_p)))
    print()
    print('const gchar *gimp_welcome_dialog_intro[] =')
    print('{')
    for p in intro_p:
      print('  "{}",'.format(p))
    print('  NULL,\n};')



#!/usr/bin/env python3

# updated to python3 - cameron

# A simple client app to the ScriptFu server.
# Usually for testing.
# CLI, a REPL: submits each entered line to the server, then prints response.

import readline, socket, sys

if len(sys.argv) < 1 or len(sys.argv) > 3:
   print >>sys.stderr, "Usage: %s <host> <port>" % sys.argv[0]
   print >>sys.stderr, "       (if omitted connect to localhost, port 10008)"
   sys.exit(1)

HOST = "localhost"
PORT = 10008

try:
    HOST = sys.argv[1]
    try:
        PORT = int(sys.argv[2])
    except IndexError:
        pass
except IndexError:
    pass

addresses = socket.getaddrinfo(HOST, PORT, socket.AF_UNSPEC, socket.SOCK_STREAM)

connected = False

for addr in addresses:
    (family, socktype, proto, canonname, sockaddr) = addr

    numeric_addr = sockaddr[0]

    if canonname:
        print ("Trying %s ('%s')." % (numeric_addr, canonname))
    else:
        print ("Trying %s." % numeric_addr)

    try:
        sock = socket.socket(family, socket.SOCK_STREAM)
        sock.connect((HOST, PORT))
        connected = True
        break
    except:
        pass

if not connected:
    print ("Failed.")
    sys.exit(1)

try:
   cmd = input("Script-Fu-Remote - Testclient\n> ")

   while len(cmd) > 0:
      my_bytes = bytearray()
      my_bytes.append(71) # G=71
      my_bytes.append((len(cmd)) >> 8)
      my_bytes.append((len(cmd)) % 0xFF)
      my_bytes.extend(bytearray(cmd,"UTF-8"))
      sock.send(my_bytes)

      data = bytearray()
      while len(data) < 4:
         data.extend(sock.recv(4 - len(data)))

      if len(data) >= 4:
         if data[0] == 71: # MAGIC_BYTE 'G'=71
            l = (data[2] << 8) + data[3]
            msg = ""
            while len(msg) < l:
               msg += (sock.recv(l - len(msg))).decode("utf-8")
            if data[1]: # ERROR_BYTE
               print ("(ERR):", msg)
            else:
               print (" (OK):", msg)
            cmd = input("> ")
         else:
            print ("invalid magic: %s\n" % data)
      else:
         print ("short response: %s\n" % data)

except EOFError:
   print

sock.close()


#!/usr/bin/env python3

"""
defcheck.py -- Consistency check for the .def files.
Copyright (C) 2006  Simon Budig <simon@gimp.org>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


This is a hack to check the consistency of the .def files compared to
the respective libraries.

Invoke in the top level of the gimp source tree after compiling GIMP.
If srcdir != builddir, run it in the build directory and pass the name
of the source directory on the command-line.

Needs the tool "nm", "objdump" or "dumpbin" to work

"""

import os, sys, subprocess, shutil, glob

from os import getenv, path

def_files = (
   "libgimpbase/gimpbase.def",
   "libgimpcolor/gimpcolor.def",
   "libgimpconfig/gimpconfig.def",
   "libgimp/gimp.def",
   "libgimp/gimpui.def",
   "libgimpmath/gimpmath.def",
   "libgimpmodule/gimpmodule.def",
   "libgimpthumb/gimpthumb.def",
   "libgimpwidgets/gimpwidgets.def"
)

have_errors = 0

srcdir = None
if len(sys.argv) > 1:
   srcdir = sys.argv[1]
   if not path.exists(srcdir):
      print("Directory '%s' does not exist" % srcdir)
      sys.exit (-1)

libextension   = ".so"
command        = getenv("NM", default="nm") + " --defined-only --extern-only "
libprefix      = "lib"
platform_linux = True

if sys.platform in ['win32', 'cygwin']:
   libextension   = ".dll"
   command        = "objdump -p "
   if shutil.which("dumpbin"):
     command      = "dumpbin /EXPORTS "
     libprefix    = ""
   platform_linux = False

for df in def_files:
   directory, name = path.split (df)
   basename, extension = name.split (".")

   libname = path.join(os.getcwd(), directory, libprefix + basename + "-*" + libextension)
   matches = glob.glob(libname)
   if matches:
     libname = matches[0]
   #FIXME: This leaks to ninja stdout, which should not happen
   #print ("platform: " + sys.platform + " - extracting symbols from " + libname)

   filename = df
   if srcdir:
      filename = path.join(srcdir, df)
   try:
      defsymbols = open (filename).read ().split ()[1:]
   except IOError as message:
      print(message)
      if not srcdir:
         print("You should run this script from the toplevel source directory.")
      sys.exit (-1)

   doublesymbols = []
   for i in range (len (defsymbols)-1, 0, -1):
      if defsymbols[i] in defsymbols[:i]:
         doublesymbols.append ((defsymbols[i], i+2))

   sorterrors = ""
   sortok = True
   for i in range (len (defsymbols)-1):
      if defsymbols[i].lower() > defsymbols[i+1].lower():
         sorterrors += f"{defsymbols[i]} > {defsymbols[i+1]}\n"
         sortok = False
   sorterrors = sorterrors.split(sep='\n')

   status, nm = subprocess.getstatusoutput (command + libname)
   if status != 0:
      print("trouble reading {} - has it been compiled?".format(libname))
      print(nm)
      have_errors = -1
      continue

   nmsymbols = ""
   if platform_linux:
      nmsymbols = nm

   elif not shutil.which("dumpbin"): # Windows MSYS2
      # remove parts of objdump output we don't need: anything up to a few lines
      # after Export Table: ' Ordinal      RVA  Name'

      objnm = nm.split(sep='\n')

      found = False
      nmsymbols = ""
      for s in objnm:
         if "Ordinal   Hint Name" in s or " Ordinal      RVA  Name" in s:
            found = True
         elif found:
            s = s.strip()
            if not s:
               break
            nmsymbols += " 0 0 " + s.split()[-1] # Keep the [2::3] logic happy
         # else: skip this line

   else: # Windows MSVC

      dbin = nm.split(sep='\n')

      found = False
      nmsymbols = ""
      for s in dbin:
         if "ordinal" in s and "hint" in s and "RVA" in s:
            found = True
         elif found and s.strip() and "Summary" not in s:
            parts = s.split()
            if len(parts) >= 4:
               nmsymbols += " 0 0 " + parts[3] # Keep the [2::3] logic happy
         # else: skip this line

   nmsymbols = nmsymbols.split()[2::3]
   nmsymbols = [s for s in nmsymbols if s[0] != '_']

   missing_defs = [s for s in nmsymbols  if s not in defsymbols and s not in exclude_symbols]
   missing_nms  = [s for s in defsymbols if s not in nmsymbols  and s not in exclude_symbols]

   if missing_defs or missing_nms or doublesymbols or not sortok:
      print()
      print("Problem found in", filename)

      if missing_defs:
         print("  the following symbols are in the library,")
         print("  but are not listed in the .def-file:")
         for s in missing_defs:
            print("     +", s)
         print()

      if missing_nms:
         print("  the following symbols are listed in the .def-file,")
         print("  but are not exported by the library.")
         for s in missing_nms:
            print("     -", s)
         print()

      if doublesymbols:
         print("  the following symbols are listed multiple times in the .def-file,")
         for s in doublesymbols:
            print("     : %s (line %d)" % s)
         print()

      if not sortok:
         print("  the .def-file is not properly sorted in the following cases")
         for s in sorterrors:
            if s != "":
               print("     * ", s)

      have_errors = -1

sys.exit (have_errors)



#!/usr/bin/env python3

#   Gimp-Python - allows the writing of Gimp plugins in Python.
#   Copyright (C) 2006  Manish Singh <yosh@gimp.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
gi.require_version('Babl', '0.1')
from gi.repository import Babl
gi.require_version('Gegl', '0.4')
from gi.repository import Gegl
gi.require_version('Gimp', '3.0')
from gi.repository import Gimp
from gi.repository import GObject
from gi.repository import GLib
from gi.repository import Gio

import sys
import traceback


def code_eval(procedure, run_mode, code, config, data):
    retval = Gimp.PDBStatusType.SUCCESS
    gerror = GLib.Error()

    if code == '-':
      code = sys.stdin.read()

    try:
      exec(code, globals())
    except Exception as error:
      retval = Gimp.PDBStatusType.CALLING_ERROR
      gerror = GLib.Error(traceback.format_exc())

    return procedure.new_return_values(retval, gerror)

class PythonEval (Gimp.PlugIn):
    ## GimpPlugIn virtual methods ##
    def do_set_i18n(self, procname):
        return True, 'gimp30-python', None

    def do_query_procedures(self):
        return ['python-fu-eval']

    def do_create_procedure(self, name):
        procedure = Gimp.BatchProcedure.new(self, name, "Python 3",
                                            Gimp.PDBProcType.PLUGIN,
                                            code_eval, None)
        procedure.set_documentation ("Evaluate Python code",
                                     "Evaluate python code under the python interpreter (primarily for batch mode)",
                                     name)

        procedure.set_attribution("Manish Singh",
                                  "Manish Singh",
                                  "2006")

        return procedure


Gimp.main(PythonEval.__gtype__, sys.argv)



#!/usr/bin/env python3
import os
import shutil
import re
import sys
import json

if not os.path.isfile("build.ninja"):
  print("\033[31m(ERROR)\033[0m: Script called standalone. This script should be only called from build systems.")
  sys.exit(1)


# This .py script should not even exist
# Ideally meson should take care of it automatically.
# See: https://github.com/mesonbuild/meson/issues/12977
if os.getenv("MESON_BUILD_ROOT", False):
  with open("meson-info/intro-installed.json", "r") as f:
    build_installed = json.load(f)
  for build_bin, installed_bin in build_installed.items():
    if build_bin.endswith((".dll", ".exe")):
      #Clang generates the .pdb on binary build dir
      pdb_debug = os.path.splitext(build_bin)[0] + ".pdb"
      #GCC generates the .pdb on build root dir
      if not os.path.isfile(pdb_debug):
        pdb_debug = os.path.join(os.getenv("MESON_BUILD_ROOT"), os.path.splitext(os.path.basename(build_bin))[0] + ".pdb")
      install_dir = os.path.dirname(installed_bin)
      if os.path.isfile(pdb_debug):
        if not os.getenv("MESON_INSTALL_DRY_RUN"):
          print(f"Installing {pdb_debug} to {install_dir}")
          shutil.copy2(pdb_debug, install_dir)

# This 'else:' part is injected by 1_build-deps-msys2.ps1 when
# we build some dependency that uses Cmake. Like meson,
# it also do not auto install .pdb files by default.
else:
  for build_root, dirs, files in os.walk(os.getcwd()):
    for file in files:
      if file.endswith('_install.cmake'):
        with open(os.path.join(build_root, file), "r") as f:
          content = f.read()
        installed_root = re.search(r'set\s*\(\s*CMAKE_INSTALL_PREFIX\s*"([^"]+)"\s*\)', content).group(1)
        for build_bin_line in re.findall(r'file\([^\)]*FILES\s+"[^"]+\.(?:dll|exe)"[^\)]*\)', content):
          build_bin = re.search(r'FILES\s+"([^"]+\.(?:dll|exe))"', build_bin_line).group(1)
          pdb_debug = os.path.splitext(build_bin)[0] + ".pdb"
          install_dir = os.path.join(installed_root, re.search(r'DESTINATION\s+"\$\{CMAKE_INSTALL_PREFIX\}(/[^"\s]+)"', build_bin_line).group(1).lstrip('/'))
          if os.path.isfile(pdb_debug):
            print(f"-- Installing: {install_dir}/{os.path.basename(pdb_debug)}")
            if not os.path.exists(install_dir):
              os.makedirs(install_dir)
            shutil.copy2(pdb_debug, install_dir)



#!/usr/bin/env python3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
gi.require_version('Gimp', '3.0')
from gi.repository import Gimp
from gi.repository import GObject
from gi.repository import GLib
from gi.repository import Gio
import sys

def N_(message): return message
def _(message): return GLib.dgettext(None, message)


def make_gradient(palette, num_segments, num_colors):

    # name the gradient same as the source palette
    # For now, the name of a resource is the same as the ID
    palette_name = palette.get_name()
    gradient = Gimp.Gradient.new(palette_name)
    # assert gradient is valid but is has only one segment
    assert gradient.get_number_of_segments() == 1

    # split one segment into desired count
    # index is zero-based
    gradient.segment_range_split_uniform(0, 0, num_segments)

    palette_colors = palette.get_colors()
    for color_number in range(0, num_segments):
        if color_number == num_colors - 1:
            color_number_next = 0
        else:
            color_number_next = color_number + 1
        color_left = palette_colors[color_number]
        color_right = palette_colors[color_number_next]
        gradient.segment_set_left_color(color_number, color_left)
        gradient.segment_set_right_color(color_number, color_right)

    # Side effects on the context. Probably not what most would expect.
    Gimp.context_set_gradient(gradient)
    return gradient

def run(procedure, config, data):
    # Get the parameters
    run_mode = config.get_property("run-mode")

    if run_mode == Gimp.RunMode.INTERACTIVE:
        gi.require_version('GimpUi', '3.0')
        from gi.repository import GimpUi
        gi.require_version('Gtk', '3.0')
        from gi.repository import Gtk

        GimpUi.init(procedure.get_name())
        dialog = GimpUi.ProcedureDialog(procedure=procedure, config=config)

        # Add palette button
        config.set_property("palette", None)
        dialog.fill (["palette"])

        if not dialog.run():
            dialog.destroy()
            return procedure.new_return_values(Gimp.PDBStatusType.CANCEL, GLib.Error())

        dialog.destroy()

    palette = config.get_property("palette")
    if palette is None or not palette.is_valid():
        if palette is not None:
          sys.stderr.write(f'Invalid palette id: {palette.get_id()}\n')
          sys.stderr.write('This should not happen. Please report to GIMP project.\n')
          sys.stderr.write('Falling back to context palette instead.\n')
        palette = Gimp.context_get_palette()
        config.set_property("palette", palette)

    num_colors = palette.get_color_count()

    if procedure.get_name() == 'python-fu-palette-to-gradient':
        num_segments = num_colors - 1
    else: # 'python-fu-palette-to-gradient-repeating'
        num_segments = num_colors
    gradient = make_gradient(palette, num_segments, num_colors)

    # XXX: for the error parameter, we want to return None.
    # Unfortunately even though the argument is (nullable), pygobject
    # looks like it may have a bug. So workaround is to just set a
    # generic GLib.Error() since anyway the error won't be process with
    # GIMP_PDB_SUCCESS status.
    # See pygobject#351
    retval = procedure.new_return_values(Gimp.PDBStatusType.SUCCESS, GLib.Error())

    # XXX: I don't try to get the GValue with retval.index(1) because it
    # actually return a string (cf. pygobject#353). Just create a new
    # GValue and replace the default one with this one.
    # This comment dates from when resources are strings:
    # value = GObject.Value(GObject.TYPE_STRING, gradient)

    value = GObject.Value(Gimp.Gradient, gradient)
    retval.remove(1)
    retval.insert(1, value)

    return retval

class PaletteToGradient (Gimp.PlugIn):
    ## GimpPlugIn virtual methods ##
    def do_set_i18n(self, procname):
        return True, 'gimp30-python', None

    def do_query_procedures(self):
        return ['python-fu-palette-to-gradient',
                'python-fu-palette-to-gradient-repeating']

    def do_create_procedure(self, name):
        procedure = Gimp.Procedure.new(self, name,
                                       Gimp.PDBProcType.PLUGIN,
                                       run, None)
        if name == 'python-fu-palette-to-gradient':
            procedure.set_menu_label(_("Palette to _Gradient"))
            procedure.set_documentation(_("Create a gradient using colors from the palette"),
                                        _("Create a new gradient using colors from the palette."),
                                        "")
        elif name == 'python-fu-palette-to-gradient-repeating':
            procedure.set_menu_label(_("Palette to _Repeating Gradient"))
            procedure.set_documentation(_("Create a repeating gradient using colors from the palette"),
                                        _("Create a new repeating gradient using colors from the palette."),
                                        "")
        else:
            procedure = None

        if procedure is not None:
            procedure.set_attribution("Carol Spears, reproduced from previous work by Adrian Likins and Jeff Trefftz",
                                      "Carol Spears", "2006")

            procedure.add_enum_argument ("run-mode", _("Run mode"),
                                         _("The run mode"), Gimp.RunMode,
                                         Gimp.RunMode.NONINTERACTIVE,
                                         GObject.ParamFlags.READWRITE)
            procedure.add_palette_argument ("palette", _("_Palette"),
                                            _("Palette"), True,
                                            None, True, # Default to context.
                                            GObject.ParamFlags.READWRITE)
            procedure.add_gradient_return_value ("new-gradient", _("The newly created gradient"),
                                                 _("The newly created gradient"),
                                                 GObject.ParamFlags.READWRITE)

            procedure.add_menu_path ('<Palettes>/Palettes Menu')

        return procedure

Gimp.main(PaletteToGradient.__gtype__, sys.argv)



#!/usr/bin/env python3
import os
import sys
import subprocess
from datetime import datetime

PERL = sys.argv[1]
top_srcdir = sys.argv[2]
top_builddir = sys.argv[3]

# Environment for the pdbgen.pl file.
os.environ['destdir'] = os.path.abspath(top_srcdir)
os.environ['builddir'] = os.path.abspath(top_builddir)

os.chdir(os.path.join(top_srcdir, 'pdb'))
result = subprocess.run(
  [PERL, 'pdbgen.pl', 'app', 'lib'],
  stdout=subprocess.PIPE,
)
if result.returncode == 0:
  with open(os.path.join(top_builddir, 'pdb', 'stamp-pdbgen.h'), 'w') as f:
    f.write(f"/* Generated on {datetime.now().strftime('%a %b %d %H:%M:%S %Z %Y')}. */\n")
sys.exit(result.returncode)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Allows exporting Krita's .kpl palettes
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
gi.require_version('Gimp', '3.0')
from gi.repository import Gimp
from gi.repository import GObject
from gi.repository import GLib
from gi.repository import Gio
import os, sys, tempfile, zipfile
import xml.etree.ElementTree as ET

def N_(message): return message
def _(message): return GLib.dgettext(None, message)

# Taken from file-openraster.py
def write_file_str(zfile, fname, data):
    # work around a permission bug in the zipfile library:
    # http://bugs.python.org/issue3394
    zi = zipfile.ZipInfo(fname)
    zi.external_attr = int("100644", 8) << 16
    zfile.writestr(zi, data)

def palette_kpl_export(procedure, config, data):
    success = True
    runmode = config.get_property("run-mode")

    if runmode == Gimp.RunMode.INTERACTIVE:
        gi.require_version('GimpUi', '3.0')
        from gi.repository import GimpUi
        gi.require_version('Gtk', '3.0')
        from gi.repository import Gtk

        GimpUi.init('python-fu-palette-export-as-kpl')
        dialog = GimpUi.ProcedureDialog(procedure=procedure, config=config)

        # Fill the dialog.
        config.set_property("palette", None)
        config.set_property("file", None)

        dialog.get_label ("options-label", _("Options"), False, False)
        hbox = dialog.fill_box ("options-box", ["comment", "read-only"])
        dialog.fill_frame("options-frame", "options-label", False, "options-box")

        dialog.fill (["palette", "file", "options-frame"])
        dialog.set_ok_label (_("_Export"))

        if not dialog.run():
            dialog.destroy()
            return procedure.new_return_values(Gimp.PDBStatusType.CANCEL, GLib.Error())

    palette   = config.get_property("palette")
    file      = config.get_property("file")
    comment   = config.get_property("comment")
    read_only = config.get_property("read-only")

    if file is None or file.peek_path() is None:
        error = 'No file given'
        return procedure.new_return_values(Gimp.PDBStatusType.CALLING_ERROR,
                                           GLib.Error(error))

    tempdir = tempfile.mkdtemp('python-fu-palette-export-as-kpl')

    kplfile = zipfile.ZipFile(file.peek_path() + '.tmpsave', 'w', compression=zipfile.ZIP_STORED)

    write_file_str(kplfile, 'mimetype', 'application/x-krita-palette') # must be the first file written

    # Filling in palette
    xml_palette = ET.Element('ColorSet')
    a = xml_palette.attrib
    if comment != "":
        a['comment'] = comment
    a['name'] = palette.get_name()
    a['version'] = "1.0"
    a['readonly'] = "true" if read_only == True else "false"
    columns = palette.get_columns()
    if columns == 0:
        columns = 16
    a['columns'] = str(columns)

    count = palette.get_color_count()

    row = 0
    col = 0
    for i in range(count):
        entry = ET.Element('ColorSetEntry')
        xml_palette.append(entry)

        success, name = palette.get_entry_name(i)
        if success:
            a = entry.attrib
            a['name'] = name
            a['id'] = str(i + 1)
            # TODO: Add options for different bit-depths
            a['bitdepth'] = "F32"
            a['spot'] = "false"

            color = palette.get_entry_color(i)
            # TODO: Allow for other color spaces
            pal = ET.Element('RGB')
            entry.append(pal)
            a = pal.attrib

            r, g, b, alpha = color.get_rgba()
            a['r'] = str(r)
            a['g'] = str(g)
            a['b'] = str(b)

            pos = ET.Element('Position')
            entry.append(pos)
            a = pos.attrib
            a['row'] = str(row)
            a['column'] = str(col)

            col += 1
            if col >= columns:
                col = 0
                row += 1
        else:
            break;

    if success:
        # Write everything to file
        xml = ET.tostring(xml_palette, encoding='UTF-8')
        write_file_str(kplfile, 'colorset.xml', xml)

        kplfile.close()
        os.rmdir(tempdir)
        if os.path.exists(file.peek_path()):
            os.remove(file.peek_path()) # win32 needs that
        os.rename(file.peek_path() + '.tmpsave', file.peek_path())

        return procedure.new_return_values(Gimp.PDBStatusType.SUCCESS, GLib.Error())
    else:
        kplfile.close()
        os.rmdir(tempdir)
        return procedure.new_return_values(Gimp.PDBStatusType.EXECUTION_ERROR,
                                           GLib.Error('File exporting failed: {}'.format(file.get_path())))

class PaletteExportAsKPL (Gimp.PlugIn):
    ## GimpPlugIn virtual methods ##
    def do_set_i18n(self, procname):
        return True, 'gimp30-python', None

    def do_query_procedures(self):
        return [ 'python-fu-palette-export-as-kpl' ]

    def do_create_procedure(self, name):
        procedure = Gimp.Procedure.new(self, name,
                                       Gimp.PDBProcType.PLUGIN,
                                       palette_kpl_export, None)
        if name == 'python-fu-palette-export-as-kpl':
            procedure.set_documentation (_("Export palette as Krita .kpl"),
                                         _("Export palette as Krita .kpl"),
                                         name)
            procedure.set_menu_label(_("_Krita palette..."))
            procedure.set_attribution("Alx Sa",
                                      "(c) GPL V3.0 or later",
                                      "2025")
            procedure.add_menu_path('<Palettes>/Palettes Menu/Export as')

            procedure.add_enum_argument ("run-mode", _("Run mode"),
                                         _("The run mode"), Gimp.RunMode,
                                         Gimp.RunMode.NONINTERACTIVE,
                                         GObject.ParamFlags.READWRITE)
            procedure.add_palette_argument ("palette", _("_Palette to export"),
                                             "", True,
                                             None, True, # Default to context.
                                             GObject.ParamFlags.READWRITE)
            procedure.add_file_argument ("file", _("_File (.kpl)"), "",
                                         Gimp.FileChooserAction.SAVE,
                                         False, None, GObject.ParamFlags.READWRITE)
            procedure.add_string_argument ("comment", _("Commen_t"),
                                           _("Optional comment"), "",
                                           GObject.ParamFlags.READWRITE)
            procedure.add_boolean_argument ("read-only",
                                            _("Re_ad only"),
                                            _("The palette will be locked on import"),
                                            False, GObject.ParamFlags.READWRITE)
        return procedure

Gimp.main(PaletteExportAsKPL.__gtype__, sys.argv)



#!/usr/bin/env python3
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
gi.require_version('Gimp', '3.0')
from gi.repository import Gimp
gi.require_version('Gegl', '0.4')
from gi.repository import Gegl
from gi.repository import GObject
from gi.repository import GLib
from gi.repository import Gio
import time
import sys

def N_(message): return message
def _(message): return GLib.dgettext(None, message)

'''
A Python plugin.
Tests GimpProcedureDialog.

Temporarily, just testing widgets for subclasses of GimpResource.
Temporarily, just testing Brush subclass of Resource.
FUTURE: For all the parameter types.
Formerly PF_ constants, now all parameters for which GimpParamSpecs exist.

Not localized, no i18n
'''


def process_args(brush, font, gradient, palette, pattern):
    '''
    Test the args are sane.
    '''
    assert brush is not None
    assert isinstance(brush, Gimp.Brush)
    id = brush.get_id()
    name = brush.get_name()
    assert id is not None
    msg = "Brush: {} (ID: {})".format(name, id)
    print(msg)
    Gimp.message(msg)

    assert font is not None
    assert isinstance(font, Gimp.Font)
    id = font.get_id()
    name = font.get_name()
    assert id is not None
    msg = "Font: {} (ID: {})".format(name, id)
    print(msg)
    Gimp.message(msg)

    assert gradient is not None
    assert isinstance(gradient, Gimp.Gradient)
    id = gradient.get_id()
    name = gradient.get_name()
    assert id is not None
    msg = "Gradient: {} (ID: {})".format(name, id)
    print(msg)
    Gimp.message(msg)

    assert palette is not None
    assert isinstance(palette, Gimp.Palette)
    id = palette.get_id()
    name = palette.get_name()
    assert id is not None
    msg = "Palette: {} (ID: {})".format(name, id)
    print(msg)
    Gimp.message(msg)

    assert pattern is not None
    assert isinstance(pattern, Gimp.Pattern)
    id = pattern.get_id()
    name = pattern.get_name()
    assert id is not None
    msg = "Pattern: {} (ID: {})".format(name, id)
    print(msg)
    Gimp.message(msg)

    return


def test_dialog(procedure, run_mode, image, drawables, config, data):
    '''
    Just a standard shell for a plugin.
    '''
    if run_mode == Gimp.RunMode.INTERACTIVE:
        gi.require_version('GimpUi', '3.0')
        from gi.repository import GimpUi

        GimpUi.init('python-fu-test-dialog')
        Gegl.init(None)
        dialog = GimpUi.ProcedureDialog(procedure=procedure, config=config)
        dialog.fill(None)
        if not dialog.run():
            dialog.destroy()
            return procedure.new_return_values(Gimp.PDBStatusType.CANCEL, GLib.Error())
        else:
            dialog.destroy()

    brush = config.get_property('brush')
    font = config.get_property('font')
    gradient = config.get_property('gradient')
    palette = config.get_property('palette')
    pattern = config.get_property('pattern')

    Gimp.context_push()

    process_args(brush, font, gradient, palette, pattern)

    Gimp.displays_flush()

    Gimp.context_pop()

    return procedure.new_return_values(Gimp.PDBStatusType.SUCCESS, GLib.Error())


class TestDialogPlugin (Gimp.PlugIn):
    # FUTURE all other Gimp classes that have GimpParamSpecs

    ## GimpPlugIn virtual methods ##
    def do_set_i18n(self, procname):
        return True, 'gimp30-python', None

    def do_query_procedures(self):
        return [ 'python-fu-test-dialog' ]

    def do_create_procedure(self, name):
        procedure = Gimp.ImageProcedure.new(self, name,
                                            Gimp.PDBProcType.PLUGIN,
                                            test_dialog, None)
        procedure.set_sensitivity_mask (Gimp.ProcedureSensitivityMask.NO_IMAGE)
        procedure.set_documentation ("Test dialog",
                                     "Test dialog",
                                     name)
        procedure.set_menu_label("Test dialog...")
        procedure.set_attribution("Lloyd Konneker",
                                  "Lloyd Konneker",
                                  "2022")
        # Top level menu "Test"
        procedure.add_menu_path ("<Image>/Filters/Development/Demos")

        procedure.add_brush_argument ("brush", "_Brush", "Brush", True,
                                      Gimp.Brush.get_by_name ("2. Hardness 025"),
                                      False,
                                      GObject.ParamFlags.READWRITE)
        procedure.add_font_argument ("font", "_Font", "Font", True,
                                     Gimp.Font.get_by_name ("Serif"),
                                     False,
                                     GObject.ParamFlags.READWRITE)
        procedure.add_gradient_argument ("gradient", "_Gradient",
                                         "Gradient", True,
                                         Gimp.Gradient.get_by_name ("Incandescent"),
                                         False,
                                         GObject.ParamFlags.READWRITE)
        procedure.add_palette_argument ("palette", "_Palette",
                                        "Palette", True,
                                        Gimp.Palette.get_by_name ("Default"),
                                        False,
                                        GObject.ParamFlags.READWRITE)
        procedure.add_pattern_argument ("pattern", "Pa_ttern",
                                        "Pattern", True,
                                        Gimp.Pattern.get_by_name ("Paper"),
                                        False,
                                        GObject.ParamFlags.READWRITE)

        return procedure

Gimp.main(TestDialogPlugin.__gtype__, sys.argv)




"""
Python scripts to test GimpPalette class of libgimp API.

See comments about usage and setup in test-resource-class.py

Setup: "Bears" palette is selected.

Testing context_get_palette is in test_resource_class.py
"""


"""
Create artifacts needed for testing.
"""
success, foreground_color = Gimp.context_get_foreground()
assert success


"""
Ensure setup was done.
"""
if Gimp.context_get_palette().get_id() != "Bears" :
    print("\n\n Setup improper: test requires palette Bears selected \n\n")


"""
Test a known, stock palette "Bears"
!!! Test numeric literals must change if the palette is changed.
"""
palette_bears  = Gimp.context_get_palette()
assert palette_bears.get_id() == "Bears"
assert palette_bears.get_color_count() == 256
assert palette_bears.get_columns() == 0
colors = palette_bears.get_colors()
assert len(colors) == 256
# color of first entry is as expected
success, color = palette_bears.get_entry_color(0)
assert success
assert color.r == 0.03137254901960784
assert color.g == 0.03137254901960784
assert color.b == 0.03137254901960784


"""
Test permissions on stock palette.
"""
# Stock palette is not editable
assert not palette_bears.is_editable()
# Stock palette cannot set columns
assert not palette_bears.set_columns(4)
# Stock palette cannot add entry
success, index = palette_bears.add_entry("foo", foreground_color)
assert success == False
# Stock palette cannot delete entry
assert not palette_bears.delete_entry(0)


"""
Test new palette.
"""
palette_new = Gimp.Palette.new("Test New Palette")

# A new palette has the requested name.
# Assuming it does not exist already because the tester followed setup correctly
assert palette_new.get_id() == "Test New Palette"

# a new palette is editable
assert palette_new.is_editable() == True

# a new palette has zero colors
assert palette_new.get_color_count() == 0

# a new palette has an empty array of colors
colors =  palette_new.get_colors()
assert len(colors) == 0

# a new palette displays with the default count of columns
assert palette_new.get_columns() == 0

# You can set columns even when empty of colors
success = palette_new.set_columns(12)
assert success == True
assert palette_new.get_columns() == 12

# Cannot set columns > 64
# Expect a gimp error dialog "Calling error...out of range"
success = palette_new.set_columns(65)
assert not success
# after out of range, still has the original value
assert palette_new.get_columns() == 12

# Cannot set columns to negative number
# TODO

"""
Test add entry
"""

# You can add entries to a new palette
success, index = palette_new.add_entry("", foreground_color)
assert success == True
# The first index is 0
assert index == 0
# After adding an entry, color count is incremented
assert palette_new.get_color_count() == 1
# The color of the new entry equals what we passed
success, color = palette_new.get_entry_color(0)
assert success == True
assert foreground_color.r == color.r


# You can add the same color twice
success, index = palette_new.add_entry("", foreground_color)
assert success == True
assert index == 1
assert palette_new.get_color_count() == 2

# An added entry for which the provided name was empty string is empty string
# TODO C code seems to suggest that it should be named "Untitled"
success, name = palette_new.get_entry_name(1)
assert success
assert name == ""

"""
Test delete entry
"""
assert palette_new.delete_entry(1) == True
# Delete entry decrements the color count
assert palette_new.get_color_count() == 1
# A deleted entry no longer is gettable
success, name = palette_new.get_entry_name(1)
assert not success
assert name == None


"""
Test that an instance that we created in previous test runs persists.
"""
palette_persisted = Gimp.Palette()
palette_persisted.set_property("id", "Test New Palette")
assert palette_persisted.get_color_count() == 1


"""
Test invalid palette

Rare.  We do not envision plugins to be constructing Gimp.Palette using the class constructor.
(Only using the new() method.)
We only envision plugins getting a Gimp.Palette from a dialog or from Gimp.Context.

Expect error dialog "A plugin is trying to use a resource that is no longer installed."
"""
palette_invalid = Gimp.Palette()    # class constructor in Python
palette_invalid.set_property("id", "invalid name")
assert palette_invalid.is_editable() == True



#!/usr/bin/env python3
import sys
import os
import re

types_associations_dict = {}

#Read file loading plug-ins sourcecode
source_files = sys.argv[3:]
for source_file in source_files:
  try:
    with open(source_file, 'r', encoding='utf-8') as f:
      content = f.read()
  except Exception as e:
    sys.stderr.write(f"(WARNING): Unable to open file {source_file}: {e}\n")
    continue

  #Parse MIME types or extensions declared in the sourcecode
  source_file_ext = os.path.splitext(source_file)[1].lower()
  if source_file_ext == '.c':
    if "LOAD_PROC" not in content and "load_procedure" not in content:
      continue
    proc_regex  = r'gimp_(load|vector_load)_procedure_new'
    mime_regex  = r'gimp_file_procedure_set_mime_types\s*\([^,]+,\s*"([^"]+)"'
    ext_regex   = r'gimp_file_procedure_set_extensions\s*\([^,]+,\s*"([^"]+)"'
    label_regex = r'gimp_procedure_set_menu_label\s*\([^,]+,\s*_\("([^"]+)"\)\s*\)'
  elif source_file_ext == '.py':
    if "LoadProcedure" not in content:
      continue
    proc_regex  = r'(LoadProcedure|VectorLoadProcedure)\.new'
    mime_regex  = r'procedure\.set_mime_types\s*\(\s*"([^"]+)"\s*\)'
    ext_regex   = r'procedure\.set_extensions\s*\(\s*"([^"]+)"\s*\)'
    label_regex = r'procedure\.set_menu_label\s*\(\s*_\("([^"]+)"\)\s*\)'
  else:
    continue
  for proc_match in re.finditer(proc_regex, content):
    #proc_name = proc_match.group(1).strip()
    #key = (source_file, f"{proc_name}_{proc_match.start()}")
    key = (source_file, proc_match.start())
    if key not in types_associations_dict:
      types_associations_dict[key] = {"mime": set(), "extensions": set(), "label": set()}
    mime_match = re.search(mime_regex, content[proc_match.end():])
    if mime_match:
      #(Take care of mimes separated by commas)
      for mime in mime_match.group(1).split(','):
        trimmed_mime = mime.strip()
        if trimmed_mime:
          types_associations_dict[key]["mime"].add(trimmed_mime)
    ext_match = re.search(ext_regex, content[proc_match.end():])
    if ext_match:
      #(Take care of extensions separated by commas)
      for ext in ext_match.group(1).split(','):
        trimmed_ext = ext.strip()
        if trimmed_ext:
          types_associations_dict[key]["extensions"].add(trimmed_ext)
    label_match = re.search(label_regex, content[proc_match.end():])
    if label_match:
      types_associations_dict[key]["label"] = label_match.group(1).strip()

mode = sys.argv[1]
if mode == "--mime":
  #Output string with the parsed MIME types
  all_mime = set()
  for values in types_associations_dict.values():
    all_mime.update(values["mime"])
  print(";".join(sorted(all_mime)))
elif mode == "--association":
  #Create list with the parsed extensions
  output_file = sys.argv[2]
  all_ext = set()
  for values in types_associations_dict.values():
    all_ext.update(values["extensions"])
  try:
    with open(output_file, 'w', encoding='utf-8') as outf:
      outf.writelines(f"{assoc}\n" for assoc in sorted(all_ext))
  except Exception as e:
    sys.stderr.write(f"(ERROR): When writing output file {output_file}: {e}\n")
    sys.exit(1)
elif mode == "--pseudo-uti":
  output_file = sys.argv[2]
  try:
    with open(output_file, 'w', encoding='utf-8') as outf:
      for (source_file, proc_key), values in types_associations_dict.items():
        extensions = sorted(values["extensions"])
        mimes = sorted(values["mime"])
        if not extensions and not mimes:
          continue

        outf.write("<dict>\n")
        if extensions:
          outf.write("  <key>CFBundleTypeExtensions</key>\n")
          outf.write("  <array>\n")
          for ext in extensions:
            outf.write(f"    <string>{ext}</string>\n")
            outf.write(f"    <string>{ext.upper()}</string>\n")
          outf.write("  </array>\n")

        if mimes:
          outf.write("  <key>CFBundleTypeMIMETypes</key>\n")
          outf.write("  <array>\n")
          for mime in mimes:
            outf.write(f"    <string>{mime}</string>\n")
          outf.write("  </array>\n")

        outf.write("  <key>CFBundleTypeIconFile</key>\n")
        outf.write("  <string>fileicon</string>\n")

        if "label" in values:
          outf.write("  <key>CFBundleTypeName</key>\n")
          outf.write(f"  <string>{values['label']}</string>\n")
        elif extensions:
          outf.write("  <key>CFBundleTypeName</key>\n")
          outf.write(f"  <string>{extensions[0].upper()} file</string>\n")

        outf.write("  <key>CFBundleTypeRole</key>\n")
        outf.write("  <string>Viewer</string>\n")

        outf.write("  <key>LSHandlerRank</key>\n")
        outf.write("  <string>Default</string>\n")
        outf.write("</dict>\n")
  except Exception as e:
    sys.stderr.write(f"(ERROR): When writing pseudo-UTI file {output_file}: {e}\n")
    sys.exit(1)



#!/usr/bin/python3

################################################################################
# Small python script to retrieve DLL or DYLIB dependencies with object dumper
################################################################################

################################################################################
# Usage example
#
# python3 lib_bundle.py /path/to/run.exe /winenv/ /path/install
#
# In this case, the DLL dependencies for executable run.exe will be extracted
# and copied into /path/install/bin folder. To copy the DLL, the root path to
# Windows environnment should be passed, here /winenv/ (usually MSYSTEM_PREFIX).
#
# python3 lib_bundle.py /path/to/run /macOSenv/ /path/install
#
# In this case, the DYLIB dependencies for executable run will be extracted
# and copied into /path/install/Frameworks folder. To copy the DYLIB,
# the root path to macOS environnment should be passed, here /macOSenv/ (usually /opt/local).

import argparse
import os
import subprocess
import re
import shutil
import sys
import struct

################################################################################
# Global variables

# Sets for executable and system DLL/DYLIB
dlls = set()
sys_dlls = set()

# Previously done DLLs/DYLIBs in previous runs
done_dlls = set()

# Common paths
bindir = 'bin'

# Platform mode (detected at runtime)
is_macos = False

################################################################################
# Functions

# Main function
def main(binary, srcdirs, destdir, debug, dll_file):
  global done_dlls
  global is_macos
  try:
    if dll_file is not None:
      with open(dll_file, 'r') as f:
        done_dlls = { line.strip() for line in f if len(line.strip()) != 0 }
  except FileNotFoundError:
    pass

  #sys.stdout.write("{} (INFO): searching for dependencies of {} in {}.\n".format(os.path.basename(__file__),
  #                                                                               binary, ', '.join(srcdirs)))
  find_dependencies(os.path.abspath(binary), srcdirs)
  if debug in ['debug-only', 'debug-run']:
    if debug == 'debug-only':
      print("Running in debug-only mode (no DLL/DYLIB moved) for '{}'".format(binary))
    else:
      print("Running with debug output for '{}'".format(binary))

    if len(dlls) > 0:
      sys.stdout.write("Needed DLLs/DYLIBs:\n\t- ")
      sys.stdout.write("\n\t- ".join(list(dlls)))
      print()
    if len(sys_dlls) > 0:
      sys.stdout.write('System DLLs/DYLIBs:\n\t- ')
      sys.stdout.write("\n\t- ".join(sys_dlls))
      print()
    removed_dlls = sys_dlls & dlls
    if len(removed_dlls) > 0:
      sys.stdout.write('Non installed DLLs/DYLIBs:\n\t- ')
      sys.stdout.write("\n\t- ".join(removed_dlls))
      print()
    installed_dlls = dlls - sys_dlls
    if len(installed_dlls) > 0:
      sys.stdout.write('Installed DLLs/DYLIBs:\n\t- ')
      sys.stdout.write("\n\t- ".join(installed_dlls))
      print()

    if debug == 'debug-run':
      copy_dlls(dlls - sys_dlls, srcdirs, destdir)
  else:
    copy_dlls(dlls - sys_dlls, srcdirs, destdir)

  if dll_file is not None:
    with open(dll_file, 'w') as f:
      f.write("\n".join(set.union(done_dlls, dlls, sys_dlls)))

def find_dependencies(obj, srcdirs):
  '''
  List DLLs/DYLIBs of an object file in a recursive way.
  '''
  global is_macos
  if obj in set.union(done_dlls, sys_dlls):
    # Already processed, either in a previous run of the script
    # (done_dlls) or in this one.
    return True

  if not os.path.isabs(obj):
    for srcdir in srcdirs:
      bindir = 'bin'
      if is_macos:
        bindir = 'lib'
      abs_dll = os.path.join(srcdir, bindir, obj)
      abs_dll = os.path.abspath(abs_dll)
      if find_dependencies(abs_dll, srcdirs):
        return True
    else:
      # Found in none of the srcdirs: consider it a system DLL/DYLIB
      sys_dlls.add(os.path.basename(obj))
      return False
  elif os.path.exists(obj):
    # If DLL/DYLIB exists, extract dependencies.
    objdump = None

    try:
      with open(obj, 'rb') as f:
        f.seek(0x3C)
        pe_offset = struct.unpack('<I', f.read(4))[0]
        f.seek(pe_offset)
        pe_sig = f.read(4)
        if pe_sig == b'PE\0\0':
          f.seek(pe_offset + 24)
          file_type = struct.unpack('<H', f.read(2))[0]
          if file_type == 0x20b:
            objdump = 'x86_64-w64-mingw32-objdump'
          elif file_type == 0x10b:
            objdump = 'i686-w64-mingw32-objdump'
          else:
            return None
        else:
          # If not PE, then check for Mach-O magic (Feed Face/Face Feed variants)
          f.seek(0) #rewind to start
          magic = f.read(4)
          if magic in [b'\xfe\xed\xfa\xce', b'\xfe\xed\xfa\xcf',
                       b'\xce\xfa\xed\xfe', b'\xcf\xfa\xed\xfe',
                       b'\xca\xfe\xba\xbe', b'\xbe\xba\xfe\xca']:
            is_macos = True
            objdump = 'no_cross_objdump'
          else:
            return None
    except Exception as e:
      sys.stderr.write("Cannot read PE/Mach-O header for {}: {}\n".format(obj, e))
      return None

    if objdump is None:
      sys.stderr.write('File type of {} unknown: {}\n'.format(obj, file_type))
      sys.exit(1)
    elif shutil.which(objdump) is None:
      # For native objdump case.
      objdump = 'objdump'
      if not is_macos and shutil.which('dumpbin'):
        objdump = 'dumpbin'
      if is_macos and shutil.which('otool'):
        objdump = 'otool'

    if shutil.which(objdump) is None:
      sys.stderr.write("Executable doesn't exist: {}\n".format(objdump))
      sys.exit(1)

    if not is_macos:
      if objdump == 'dumpbin':
        result = subprocess.run([objdump, '/DEPENDENTS', obj], stdout=subprocess.PIPE)
        out = result.stdout.decode('utf-8')
        regex = re.finditer(r'\b([\w\-.]+\.dll)\b', out, re.MULTILINE)
      else:
        result = subprocess.run([objdump, '-p', obj], stdout=subprocess.PIPE)
        out = result.stdout.decode('utf-8')
        regex = re.finditer(r"DLL Name: *(\S+.dll)", out, re.MULTILINE)
    else:
      if objdump == 'otool':
        result = subprocess.run(['otool', '-l', obj], stdout=subprocess.PIPE)
      else:
        result = subprocess.run(['objdump', '--macho', '--private-headers', obj], stdout=subprocess.PIPE)
      out = result.stdout.decode('utf-8', errors='replace')
      regex = re.finditer(r"name\s+(?:\d+\s+)?(?:.*/)?([^/\s]+\.dylib)", out, re.MULTILINE)
    # Parse lines with DLL/DYLIB Name instead of lib*.dll/lib*.dylib directly
    for match in regex:
      dll = match.group(1)
      if dll not in set.union(done_dlls, dlls, sys_dlls):
        dlls.add(dll)
        find_dependencies(dll, srcdirs)

    return True
  else:
    return False

# Copy a DLL/DYLIB set into the /destdir/bin or /destdir/Frameworks directory
def copy_dlls(dll_list, srcdirs, destdir):
  global bindir
  dest_bindir = bindir
  if is_macos:
    dest_bindir = 'Frameworks'
  destbin = os.path.join(destdir, dest_bindir)
  os.makedirs(destbin, exist_ok=True)
  if is_macos:
    abs_binary = os.path.abspath(args.bin)
    abs_destdir = os.path.abspath(destdir)
    if abs_binary.startswith(abs_destdir + os.sep):
      #check_macos_version(abs_binary)
      set_rpath(abs_binary, destbin)
  for dll in dll_list:
    if not os.path.exists(os.path.join(destbin, dll)):
      # Do not overwrite existing files.
      for srcdir in srcdirs:
        if is_macos:
          bindir = 'lib'
        full_file_name = os.path.join(srcdir, bindir, dll)
        if os.path.isfile(full_file_name):
          if is_macos and not full_file_name.startswith(("/opt/local/", "/opt/homebrew/")):
            check_macos_version(full_file_name)
          sys.stdout.write("Bundling {} to {}\n".format(full_file_name, destbin))
          shutil.copy(full_file_name, destbin)
          if is_macos:
            set_rpath(os.path.join(destbin, dll), destbin)
          break
      else:
        # This should not happen. We determined that the dll is in one
        # of the srcdirs.
        sys.stderr.write("Missing DLL/DYLIB: {}\n".format(dll))
        sys.exit(1)

def check_macos_version(binary):
  """
  Check LC_BUILD_VERSION for compatibility with MACOSX_DEPLOYMENT_TARGET
  """
  supported_minos = os.environ.get("MACOSX_DEPLOYMENT_TARGET")
  if not supported_minos:
    return

  result = subprocess.run(['otool', '-l', binary], stdout=subprocess.PIPE, check=True)
  out = result.stdout.decode('utf-8', errors='replace')

  bin_minos = re.findall(r'minos\s+([\d\.]+)', out)[0]
  if tuple(map(int, bin_minos.split('.'))) > tuple(map(int, supported_minos.split('.'))):
    sys.stderr.write(f"\033[31m(ERROR)\033[0m: {binary} requires macOS {bin_minos}, which is higher than macOS {supported_minos} which GIMP was built against.\n")
    sys.exit(1)

def set_rpath(binary, destbin=None):
  """
  Remove all LC_RPATH entries except those identical to relative destbin,
  and set it if not. Also rewrite LC_LOAD_DYLIB and LC_ID_DYLIB to use it.
  """
  result = subprocess.run(['otool', '-l', binary], stdout=subprocess.PIPE)
  out = result.stdout.decode('utf-8', errors='replace')

  # Handle LC_RPATH (only on executables as a rule)
  if (".dylib" not in binary and ".so" not in binary) or "cmd LC_RPATH" in out:
    regex = re.findall(r'path (.+?) \(offset', out)
    new_rpath = os.path.join("@executable_path", os.path.relpath(destbin, os.path.dirname(os.path.abspath(binary))))
    for old_rpath in regex:
      if old_rpath == new_rpath:
        continue
      try:
        subprocess.run(['install_name_tool', '-delete_rpath', old_rpath, binary], check=True, stderr=subprocess.DEVNULL)
        #sys.stdout.write(f"Removed LC_RPATH {old_rpath} from {binary}\n")
      except subprocess.CalledProcessError as e:
        sys.stderr.write(f"Failed to remove rpath {old_rpath} from {binary}: {e}\n")
    if new_rpath not in regex:
      try:
        subprocess.run(['install_name_tool', '-add_rpath', new_rpath, binary], check=True, stderr=subprocess.DEVNULL)
        #sys.stdout.write(f"Added LC_RPATH {new_rpath} to {binary}\n")
      except subprocess.CalledProcessError as e:
        sys.stderr.write(f"Failed to add rpath {new_rpath} to {binary}: {e}\n")

  # Handle LC_LOAD_DYLIB (on executables, shared libraries)
  regex = re.findall(r'name (.+?) \(offset', out)
  for old_dylib_path in regex:
    if old_dylib_path.startswith("/usr") or old_dylib_path.startswith("/System"):
      continue
    if ".framework" in old_dylib_path:
      new_dylib_path = os.path.join("@rpath", old_dylib_path[old_dylib_path.rfind("/", 0, old_dylib_path.find(".framework")) + 1:])
    else:
      new_dylib_path = os.path.join("@rpath", os.path.basename(old_dylib_path))
    if old_dylib_path != new_dylib_path:
      try:
        subprocess.run(['install_name_tool', '-change', old_dylib_path, new_dylib_path, binary], check=True, stderr=subprocess.DEVNULL)
        #sys.stdout.write(f"Rewrote LC_LOAD_DYLIB {old_dylib_path} -> {new_dylib_path}\n")
      except subprocess.CalledProcessError as e:
        sys.stderr.write(f"Failed to rewrite LC_LOAD_DYLIB {old_dylib_path}: {e}\n")

  # Handle LC_ID_DYLIB (only on shared libraries)
  regex = re.search(r'cmd LC_ID_DYLIB.*?\n\s*name (.+?) \(offset', out, re.DOTALL)
  if (".dylib" in binary or ".so" in binary) and regex:
    old_dylib_path = regex.group(1).strip()
    new_dylib_path = os.path.join("@rpath", os.path.basename(old_dylib_path))
    if old_dylib_path != new_dylib_path:
      try:
        subprocess.run(['install_name_tool', '-id', new_dylib_path, binary], check=True, stderr=subprocess.DEVNULL)
        #sys.stdout.write(f"Rewrote LC_ID_DYLIB {old_dylib_path} -> {new_dylib_path}\n")
      except subprocess.CalledProcessError as e:
        sys.stderr.write(f"Failed to rewrite LC_ID_DYLIB {old_dylib_path}: {e}\n")

  # Normalize signature after all the changes
  result = subprocess.run(["codesign", "-v", binary], capture_output=True, text=True)
  if "invalid signature" in result.stdout + result.stderr:
    try:
      subprocess.run(["codesign", "--sign", "-", "--force", "--preserve-metadata=entitlements,requirements,flags,runtime", binary], check=True, stderr=subprocess.DEVNULL)
      #sys.stdout.write(f"Re-signed {binary}\n")
    except subprocess.CalledProcessError as e:
      sys.stderr.write(f"Failed to ad-hoc sign {binary}: {e}\n")


if __name__ == "__main__":
  parser = argparse.ArgumentParser()
  parser.add_argument('--debug', dest='debug',
                      choices=['debug-only', 'debug-run', 'run-only'], default = 'run-only')
  parser.add_argument('--output-dll-list', dest='dll_file', action = 'store', default = None)
  parser.add_argument('bin')
  parser.add_argument('src', nargs='+')
  parser.add_argument('dest')
  args = parser.parse_args(sys.argv[1:])

  main(args.bin, args.src, args.dest, args.debug, args.dll_file)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
gi.require_version('Gimp', '3.0')
from gi.repository import Gimp
from gi.repository import GObject
from gi.repository import GLib
from gi.repository import Gio
import sys

def N_(message): return message
def _(message): return GLib.dgettext(None, message)

help_doc = r"""
Offset the colors in the palette.
Offsets and returns the given palette when it is editable,
otherwise copies the given palette and returns it.
"""

class PaletteOffset (Gimp.PlugIn):
    ## GimpPlugIn virtual methods ##
    def do_set_i18n(self, procname):
        return True, 'gimp30-python', None

    def do_query_procedures(self):
        return [ "python-fu-palette-offset" ]

    def do_create_procedure(self, name):
        procedure = Gimp.Procedure.new(self, name,
                                       Gimp.PDBProcType.PLUGIN,
                                       self.run, None)
        if name == 'python-fu-palette-offset':
            procedure.set_menu_label(_("_Offset Palette..."))
            procedure.set_documentation(_("Offset the colors in a palette"),
                                        help_doc,
                                        "")
            procedure.set_attribution("Joao S. O. Bueno Calligaris, Carol Spears",
                                      "(c) Joao S. O. Bueno Calligaris",
                                      "2004, 2006")
            procedure.add_enum_argument ("run-mode", _("Run mode"),
                                         _("The run mode"), Gimp.RunMode,
                                         Gimp.RunMode.NONINTERACTIVE,
                                         GObject.ParamFlags.READWRITE)
            procedure.add_palette_argument ("palette", _("_Palette"),
                                            _("Palette"), True,
                                            None, True, # Default to context.
                                            GObject.ParamFlags.READWRITE)
            procedure.add_int_argument ("amount", _("O_ffset"), _("Offset"),
                                        1, GLib.MAXINT, 1, GObject.ParamFlags.READWRITE)
            procedure.add_palette_return_value ("new-palette", _("The edited palette"),
                                                _("The newly created palette when read-only, otherwise the input palette"),
                                                GObject.ParamFlags.READWRITE)
            procedure.add_menu_path ('<Palettes>/Palettes Menu')
        else:
            procedure = None

        return procedure

    def run(self, procedure, config, data):
        runmode = config.get_property("run-mode")
        palette = config.get_property("palette")
        amount  = config.get_property("amount")

        if palette is None:
            palette = Gimp.context_get_palette()
            config.set_property("palette", palette)

        if not palette.is_valid():
            error = f'Invalid palette ID: {palette.get_id()}'
            return procedure.new_return_values(Gimp.PDBStatusType.CALLING_ERROR,
                                               GLib.Error(error))

        if runmode == Gimp.RunMode.INTERACTIVE:
            gi.require_version('GimpUi', '3.0')
            from gi.repository import GimpUi
            gi.require_version('Gtk', '3.0')
            from gi.repository import Gtk

            GimpUi.init ("palette-offset.py")

            dialog = GimpUi.ProcedureDialog(procedure=procedure, config=config)

            config.set_property("palette", None)
            dialog.fill(["palette", "amount"])
            if not dialog.run():
                dialog.destroy()
                return procedure.new_return_values(Gimp.PDBStatusType.CANCEL, GLib.Error())
            else:
                dialog.destroy()

            amount = config.get_property("amount")
            palette = config.get_property("palette")

        #If palette is read only, work on a copy:
        editable = palette.is_editable()
        if not editable:
            palette = palette.duplicate()

        num_colors = palette.get_color_count()
        tmp_entry_array = []
        for i in range (num_colors):
            tmp_entry_array.append  ((palette.get_entry_name(i)[1],
                                      palette.get_entry_color(i)))
        for i in range (num_colors):
            target_index = i + amount
            if target_index >= num_colors:
                target_index -= num_colors
            elif target_index < 0:
                target_index += num_colors
            palette.set_entry_name(target_index, tmp_entry_array[i][0])
            palette.set_entry_color(target_index, tmp_entry_array[i][1])

        retval = procedure.new_return_values(Gimp.PDBStatusType.SUCCESS, GLib.Error())
        value = GObject.Value(Gimp.Palette, palette)
        retval.remove(1)
        retval.insert(1, value)
        return retval

Gimp.main(PaletteOffset.__gtype__, sys.argv)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#   GIMP - The GNU Image Manipulation Program
#   Copyright (C) 1995 Spencer Kimball and Peter Mattis
#
#   test-file-plug-ins.py
#   Copyright (C) 2021-2024 Jacob Boerema
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""Plug-in to test GIMP file import plug-ins from within GIMP."""

import sys

import gi
gi.require_version('Gimp', '3.0')
from gi.repository import Gimp

from gi.repository import GLib

from gimpconfig import GimpConfig
from gimplogger import GimpLogger
from gimptestframework import GimpTestRunner, GimpExportTestRunner, AUTHORS, YEARS, VERSION
from gimpexporttests import BmpExportTests


#DEBUGGING=True
DEBUGGING=False

#PRINT_VERBOSE = True
PRINT_VERBOSE = False

LOG_APPEND = False


class PythonTest (Gimp.PlugIn):

    def __init__(self):
        Gimp.PlugIn.__init__(self)
        self.test_cfg = None
        self.log = None

    ## GimpPlugIn virtual methods ##
    def do_set_i18n(self, _name):
        # We don't support internationalization here...
        return False

    def do_query_procedures(self):
        return [ 'test-import-plug-ins',
                 'test-export-plug-ins' ]

    def do_create_procedure(self, name):

        if name == 'test-import-plug-ins':
            procedure = Gimp.ImageProcedure.new(self, name,
                                                Gimp.PDBProcType.PLUGIN,
                                                self.run_import_tests, None)
            procedure.set_image_types("*")
            procedure.set_sensitivity_mask(Gimp.ProcedureSensitivityMask.ALWAYS)
            procedure.set_menu_label('Test file _import plug-ins')
            procedure.add_menu_path('<Image>/Filters/Development/Python-Fu/')
            procedure.set_documentation('Run file import plug-in tests',
                                        'Run file import plug-in tests',
                                        name)
        elif name == 'test-export-plug-ins':
            procedure = Gimp.ImageProcedure.new(self, name,
                                                Gimp.PDBProcType.PLUGIN,
                                                self.run_export_tests, None)
            procedure.set_image_types("*")
            procedure.set_sensitivity_mask(Gimp.ProcedureSensitivityMask.ALWAYS)
            procedure.set_menu_label('Test file _export plug-ins')
            procedure.add_menu_path('<Image>/Filters/Development/Python-Fu/')
            procedure.set_documentation('Run file export plug-in tests',
                                        'Run file export plug-in tests',
                                        name)
        else:
            return None

        procedure.set_attribution(AUTHORS, #author
                                  AUTHORS, #copyright
                                  YEARS)   #year
        return procedure

    def init_logging(self):
        self.test_cfg = GimpConfig()

        self.log = GimpLogger(True, self.test_cfg.log_file, LOG_APPEND, PRINT_VERBOSE, DEBUGGING)

    def run_import_tests(self, procedure, _run_mode, _image,
                         _drawable, _config, _data):
        self.init_logging()

        if self.log.enabled:
            runner = GimpTestRunner(self.log, "import", self.test_cfg)
            runner.run_tests()

        return procedure.new_return_values(Gimp.PDBStatusType.SUCCESS, GLib.Error())

    def run_export_tests(self, procedure, _run_mode, _image,
                         _drawable, _config, _data):
        self.init_logging()

        if self.log.enabled:
            runner = GimpExportTestRunner(self.log, "export", self.test_cfg)
            if not runner:
                self.log.error("Failed to create export test runner!")
            else:
                runner.load_test_configs()

                bmp_tests = BmpExportTests("bmp", self.log)
                runner.add_test(bmp_tests)

                # Add additional tests here

                runner.run_tests()

        return procedure.new_return_values(Gimp.PDBStatusType.SUCCESS, GLib.Error())

Gimp.main(PythonTest.__gtype__, sys.argv)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#   GIMP - The GNU Image Manipulation Program
#   Copyright (C) 1995 Spencer Kimball and Peter Mattis
#
#   gimptestframework.py
#   Copyright (C) 2021-2024 Jacob Boerema
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""GIMP file plug-ins testing framework."""

import os
import configparser

import xml
from xml.etree.ElementTree import ElementTree, Element

import gi
gi.require_version('Gimp', '3.0')
from gi.repository import Gimp
from gi.repository import Gio


VERSION = "0.5"
AUTHORS = "Jacob Boerema"
YEARS   = "2021-2024"

EXPECTED_FAIL = 0
EXPECTED_OK   = 1
EXPECTED_TODO = 2

RESULT_FAIL   = 0
RESULT_OK     = 1
RESULT_CRASH  = 2


class PluginTestConfig(object):
    def __init__(self, log, path, testconfig):
        self.valid_config = False
        self.enabled = False

        self.plugin = testconfig.get('plugin-import')
        if self.plugin is None:
            self.plugin = testconfig.get('plugin')
            if self.plugin is not None:
                log.warning("Test is using deprecated 'plugin' parameter. Use 'plugin-import' instead!")
            else:
                log.error("Missing required 'plugin-import' parameter!")
                return

        tests = testconfig.get('tests')
        if tests is None:
            log.error("Missing required 'tests' parameter!")
            return
        testsuite_basename = tests + '-'
        self.tests = path + tests

        enabled = testconfig.get('enabled')
        if enabled == 'True':
            self.enabled = True

        self.extension = testconfig.get('extension','<missing>')
        self.valid_config = True

        # preliminary export testing support
        self.export_plugin = testconfig.get('plugin-export')
        self.export_tests = testconfig.get('tests-export')
        self.export_enabled = testconfig.get('enabled-export')

        self.testsuite_import = testsuite_basename + self.plugin
        if self.export_plugin is not None:
            self.testsuite_export = testsuite_basename + self.export_plugin


class ConfigLoader(object):
    def __init__(self, log, config_path, config_file):
        self.config_path = config_path
        self.config_file = config_file
        log.debug(f"Using config file: {self.config_path}{self.config_file}")
        self.tests = []
        self.export_tests = []
        self.problems = 0
        self.disabled = 0
        self.config = configparser.ConfigParser()
        self.config.read(self.config_path + self.config_file)

        msg  = "Available tests:"
        for key in self.config.sections():
            if key == 'main':
                pass
            else:
                # Add test config
                test = PluginTestConfig(log, self.config_path, self.config[key])
                if test.valid_config:
                    self.tests.append(test)
                    msg += f"\nPlugin: {test.plugin}, test config: {test.tests}, enabled: {test.enabled}"
                    if not test.enabled:
                        self.disabled += 1
                else:
                    log.warning(f"\nSkipping invalid config for {key}!")
                    self.problems += 1

                if test.export_plugin is not None and test.export_tests is not None:
                    log.info(f"Export test found for {key}")
                    self.export_tests.append(test)

        log.gimp_verbose_message(msg)

class FileLoadTest(object):
    def __init__(self, file_load_plugin_name, image_type, data_path, log, testsuite):
        self.log = log
        self.testsuite = testsuite
        self.data_root = data_path
        self.plugin_name = file_load_plugin_name
        self.image_type  = image_type

        self.unexpected_success_images = []
        self.unexpected_failure_images = []

        self.failure_reason = "unknown"

        self.total_tests  = 0
        self.total_failed = 0
        self.total_todo   = 0
        self.total_ok     = 0
        self.total_crash  = 0

    def run_file_load(self, image_file, expected):
        if not os.path.exists(image_file):
            msg = "Regression loading " + image_file + ". File does not exist!"
            self.failure_reason = msg
            self.log.error("--> " + msg)
            return RESULT_FAIL

        pdb_proc   = Gimp.get_pdb().lookup_procedure(self.plugin_name)
        if pdb_proc is None:
            msg = "Plug-in procedure '" + self.plugin_name + "' not found!"
            self.failure_reason = msg
            self.log.error("--> " + msg)
            return RESULT_FAIL
        pdb_config = pdb_proc.create_config()
        pdb_config.set_property('run-mode', Gimp.RunMode.NONINTERACTIVE)
        pdb_config.set_property('file', Gio.File.new_for_path(image_file))
        result = pdb_proc.run(pdb_config)
        status = result.index(0)
        img = result.index(1)
        if (status == Gimp.PDBStatusType.SUCCESS and img is not None and img.is_valid()):
            self.log.info ("Loading succeeded for " + image_file)
            img.delete()
            if expected == EXPECTED_FAIL:
                self.unexpected_success_images.append(image_file)
                msg = "Regression loading " + image_file + ". Loading unexpectedly succeeded."
                self.failure_reason = msg
                self.log.error("--> " + msg)
                return RESULT_FAIL
            elif expected == EXPECTED_TODO:
                self.unexpected_success_images.append(image_file)
                msg = "Loading unexpectedly succeeded for test marked TODO. Image: " + image_file
                self.failure_reason = msg
                self.log.error("--> " + msg)
                return RESULT_FAIL

            return RESULT_OK
        else:
            self.log.info ("Loading failed for " + image_file)
            if status == Gimp.PDBStatusType.CALLING_ERROR:
                # A calling error indicates the plug-in crashed, which should
                # always be considered failure!
                if expected == EXPECTED_OK:
                    self.unexpected_failure_images.append(image_file)
                elif expected == EXPECTED_TODO:
                    self.unexpected_success_images.append(image_file)
                msg = "Plug-in crashed while loading " + image_file + "."
                self.failure_reason = msg
                self.log.error("--> " + msg)
                return RESULT_CRASH
            elif expected == EXPECTED_OK:
                self.unexpected_failure_images.append(image_file)
                msg = "Regression loading " + image_file + ". Loading unexpectedly failed."
                self.failure_reason = msg
                self.log.error("--> " + msg)
                return RESULT_FAIL
            return RESULT_OK

    def load_test_images(self, test_images):
        test_list = []

        if not os.path.exists(test_images):
            msg = "Path does not exist: " + test_images
            self.log.error(msg)
            return None

        with open(test_images, encoding='UTF-8') as f:
            try:
                content = f.readlines()
            except UnicodeDecodeError as err:
                self.log.error(f"Invalid encoding for {test_images}: {err}")
                return None

            # strip whitespace
            content = [x.strip() for x in content]

        for line in content:
            data = line.split(",")
            data[0] = data[0].strip()
            if len(data) > 1:
                expected_str = data[1].strip()
            else:
                expected_str = 'EXPECTED_OK'
                data.append(expected_str)

            if expected_str == 'EXPECTED_FAIL':
                data[1] = EXPECTED_FAIL
            elif expected_str == 'EXPECTED_TODO':
                data[1] = EXPECTED_TODO
            elif expected_str == 'SKIP':
                continue
            else:
                # Assume that anything else is supposed to load OK
                data[1] = EXPECTED_OK
            test_list.append(data)

        return test_list

    def run_tests(self, image_folder, test_images, test_description):
        test_fail  = 0
        test_ok    = 0
        test_todo  = 0
        test_crash = 0
        test_total = 0
        test_images_list = self.load_test_images(test_images)
        if not test_images_list:
            # Maybe we should create another type of test failure (test_filesystem?)
            test_total += 1
            test_fail  += 1
            test_crash += 1
            self.total_failed += test_fail
            self.total_crash  += test_crash
            msg = f"No images found for '{test_description}'.'"
            self.log.error(msg)
            return
        test_total = len(test_images_list)

        for imgfile, expected in test_images_list:
            test_result = self.run_file_load(self.data_root + image_folder + imgfile, expected)
            el = Element("testcase")
            el.set('classname', self.testsuite.get('classname', 'unknown'))
            el.set('name', image_folder + imgfile)
            el.set('file', imgfile)
            self.testsuite.append(el)
            if test_result == RESULT_OK:
                test_ok += 1
            else:
                if test_result == RESULT_FAIL:
                    test_fail += 1
                elif test_result == RESULT_CRASH:
                    test_crash += 1
                    test_fail  += 1
                else:
                    test_crash += 1
                    test_fail  += 1
                    msg = "Invalid test result value: " + str(test_result)
                    self.log.error(msg)

                # Add failed testcase
                failure = Element("failure")
                failure.set('type', 'failure')
                failure.text = self.failure_reason
                el.append(failure)

            if expected == EXPECTED_TODO:
                test_todo += 1

        result_msg = "Test: " + test_description + "\n"
        if test_crash > 0:
            result_msg += "Number of plug-in crashes: " + str(test_crash) + "\n"

        if test_fail > 0:
            result_msg += "Failed " + str(test_fail) + " of " + str(test_total) + " tests"
            if test_todo > 0:
                result_msg += ", and " + str(test_todo) + " known test failures."
        elif test_todo > 0:
            result_msg += "All " + str(test_total) + " tests succeeded, but there are " + str(test_todo) + " known test failures."
        else:
            result_msg += "All " + str(test_total) + " tests succeeded."

        self.log.info("\n--- results ---")
        self.log.info(result_msg)
        if ((test_fail > 0 or test_todo > 0) and self.log.interactive):
            Gimp.message (result_msg)

        self.total_tests  += test_total
        self.total_todo   += test_todo
        self.total_failed += test_fail
        self.total_ok     += test_ok
        self.total_crash  += test_crash

    def show_results_total(self):
        msg  = "\n----- Test result totals -----"
        msg += f"\nTotal {self.image_type} tests: {self.total_tests}"
        if self.total_crash > 0:
            msg += f"\nTests crashed: {self.total_crash}"
        msg += f"\nTests failed: {self.total_failed}"
        if self.total_crash > 0:
            msg += f" (including crashes)"
        msg += f"\nTests todo: {self.total_todo}"
        self.log.message(msg)

        msg = ""
        for img in self.unexpected_success_images:
            msg += f"{img}\n"
        if len(msg) > 0:
            msg = "\nImages that unexpectedly loaded:\n" + msg
            self.log.message(msg)
        msg = ""
        for img in self.unexpected_failure_images:
            msg += f"{img}\n"
        if len(msg) > 0:
            msg = "\nImages that unexpectedly failed to load:\n" + msg
            self.log.message(msg)

class RunTests(object):
    def __init__(self, test, config_path, data_path, log, testsuite):
        self.test_count = 0
        self.regression_count = 0
        if os.path.exists(test.tests):
            self.plugin_test = FileLoadTest(test.plugin, test.extension, data_path, log, testsuite)
            cfg = configparser.ConfigParser()
            cfg.read(test.tests)
            for subtest in cfg.sections():
                if not 'description' in cfg[subtest]:
                    description = "Missing description for " + test.extension
                else:
                    description = cfg[subtest]['description']
                enabled = True
                if 'enabled' in cfg[subtest]:
                    if cfg[subtest]['enabled'] == 'False':
                        enabled = False
                folder = cfg[subtest]['folder']
                files = self.plugin_test.data_root + folder + cfg[subtest]['files']
                if enabled:
                    self.plugin_test.run_tests(folder, files, description)
                else:
                    log.info(f"Testing is disabled for: {description}.")

            self.plugin_test.show_results_total()
            self.test_count = self.plugin_test.total_tests
            self.regression_count = self.plugin_test.total_failed
        else:
            self.plugin_test = None
            log.error("Test path " + test.tests + " does not exist!")
            self.regression_count = 1

    def get_unexpected_success_regressions(self):
        return self.plugin_test.unexpected_success_images

    def get_unexpected_failure_regressions(self):
        return self.plugin_test.unexpected_failure_images

    def get_todo_count(self):
        return self.plugin_test.total_todo

    def get_crash_count(self):
        return self.plugin_test.total_crash


class GimpTestRunner(object):
    def __init__(self, log, test_type, test_cfg):
        self.log = log
        self.test_type = test_type
        self.test_cfg = test_cfg

        self.tests_total = 0
        self.error_total = 0
        self.todo_total  = 0
        self.crash_total = 0

        self.unexpected_success_images = []
        self.unexpected_failure_images = []

        el = Element("testsuites")
        el.set('name', test_type)
        self.xml_tree = ElementTree(el)

    def print_header(self, test_type):
        divider = "\n--------------------------------"
        msg  = f"\nGIMP file plug-in tests version {VERSION}\n"
        #msg += f"\n{AUTHORS}, {YEARS}\n"
        msg += f"\n--- Starting {test_type} test run ---"
        msg += divider

        if self.log.verbose:
            msg += f"\nConfig: {self.test_cfg.config_folder + self.test_cfg.config_file}"
            msg += f"\nLog: {self.test_cfg.log_file}"
            msg += f"\nData path: {self.test_cfg.data_folder}"

        self.log.message(msg)

    def print_footer(self, msg):
        msg += "\n--------------------------------"
        self.log.message(msg)

    def list_regressions(self):
        regression_list = ""
        msg = ""
        extra = "\n"
        for img in self.unexpected_success_images:
            msg += f"\n{img}"
        if len(msg) > 0:
            regression_list = "\nImages that unexpectedly loaded:" + msg + extra
            extra = ""

        msg = ""
        for img in self.unexpected_failure_images:
            msg += f"\n{img}"
        if len(msg) > 0:
            regression_list += "\nImages that unexpectedly failed to load:" + msg + extra

        return regression_list

    def run_tests(self):
        self.print_header(self.test_type)

        # Load test config
        cfg = ConfigLoader(self.log, self.test_cfg.config_folder, self.test_cfg.config_file)

        root = self.xml_tree.getroot()

        # Actual tests
        for test in cfg.tests:
            if test.enabled:
                self.log.consoleinfo(f"\nTesting {test.extension} import using {test.plugin}...\n")
                el = Element("testsuite")
                el.set('name', test.testsuite_import)
                el.set('classname', test.testsuite_import)
                plugin_tests = RunTests(test, cfg.config_path, self.test_cfg.data_folder, self.log, el)
                self.tests_total += plugin_tests.test_count
                self.error_total += plugin_tests.regression_count

                el.set('tests', str(plugin_tests.test_count))
                el.set('failures', str(plugin_tests.regression_count))
                el.set('errors', '0')
                el.set('skipped', '0')
                root.append(el)

                if plugin_tests.plugin_test is not None:
                    self.todo_total  += plugin_tests.get_todo_count()
                    self.crash_total += plugin_tests.get_crash_count()
                    if plugin_tests.regression_count > 0:
                        temp = plugin_tests.get_unexpected_success_regressions()
                        if len(temp) > 0:
                            self.unexpected_success_images.extend(temp)
                        temp = plugin_tests.get_unexpected_failure_regressions()
                        if len(temp) > 0:
                            self.unexpected_failure_images.extend(temp)
                else:
                    self.crash_total += 1
                self.log.consoleinfo(f"\nFinished testing {test.extension}\n")
            else:
                self.log.consoleinfo(f"Testing {test.extension} import using {test.plugin} is disabled.")

        root.set('tests', str(self.tests_total))
        root.set('failures', str(self.error_total))
        root.set('errors', '0')

        xml.etree.ElementTree.indent(self.xml_tree)
        self.xml_tree.write(self.test_cfg.xml_results_file, 'UTF-8')

        msg  = "\n--------- Test results ---------"
        if cfg.disabled > 0:
            msg += f"\nNumber of test sections disabled: {cfg.disabled}"
        if cfg.problems > 0:
            msg += f"\nNumber of test sections skipped due to configuration errors: {cfg.problems}"
        msg += f"\nTotal number of tests executed: {self.tests_total}"
        msg += f"\nTotal number of regressions: {self.error_total}"
        if self.crash_total > 0:
            msg += f"\nTotal number of crashes: {self.crash_total}"
        if self.todo_total > 0:
            msg += f"\nTotal number of todo tests: {self.todo_total}"
        if self.error_total > 0:
            msg += "\n" + self.list_regressions()
        self.print_footer(msg)

class GimpExportTestGroup(object):
    def __init__(self, group_test_cfg, group_name, group_description,
                 input_folder, output_folder):
        self.group_test_cfg = group_test_cfg
        self.group_name = group_name
        self.group_description = group_description
        self.input_folder = input_folder
        self.output_folder = output_folder

class GimpExportTestSections(object):
    def __init__(self, test, config_path, data_path, log):
        self.test_groups = []
        self.test_count = 0
        self.regression_count = 0
        self.export_test_path = config_path + test.export_tests
        if os.path.exists(self.export_test_path):
            cfg = configparser.ConfigParser()
            cfg.read(self.export_test_path)
            for subtest in cfg.sections():
                if not 'description' in cfg[subtest]:
                    description = "Missing description for " + test.extension
                    log.warning(f"Missing description for {test.extension}")
                else:
                    description = cfg[subtest]['description']
                    log.info(f"Test: {description}")
                enabled = True
                if 'enabled' in cfg[subtest]:
                    if cfg[subtest]['enabled'] == 'False':
                        enabled = False

                folder_input = cfg[subtest]['folder-input']
                folder_output = cfg[subtest]['folder-output']
                if enabled:
                    group = GimpExportTestGroup(test, test.extension, description,
                                                folder_input, folder_output)
                    self.test_groups.append(group)
                else:
                    log.info(f"Testing is disabled for: {description}.")

        else:
            log.error("Test path " + test.export_tests + " does not exist!")
            self.regression_count = 1

class GimpExportTest(object):
    def __init__(self, group_name, log):
        self.group_name = group_name
        self.log = log

        self.base_path = None
        self.input_path = None
        self.output_path = None
        self.file_import = None
        self.file_export = None

    def _setup(self, group_config, test_data_path):
        self.base_path = test_data_path
        self.input_path = self.base_path + group_config.input_folder
        self.output_path = self.base_path + group_config.output_folder
        if self.base_path is None or self.input_path is None or self.output_path is None:
            return False

        self.file_import = group_config.group_test_cfg.plugin
        self.file_export = group_config.group_test_cfg.export_plugin
        if self.file_import is None or self.file_export is None:
            return False
        return True

    def setup(self, group_config):
        pass

    def teardown(self, group_config):
        pass

    def run_test(self, group_config):
        pass

    def execute(self, group_config, test_data_path):
        if not self._setup(group_config, test_data_path):
            self.log.error("Invalid config, can't execute test!")
            return
        self.setup(group_config)
        self.run_test(group_config)
        self.teardown(group_config)

class GimpExportTestRunner(GimpTestRunner):
    def __init__(self, log, test_type, test_cfg):
        self.cfg = None
        self.log = log
        self.test_type = test_type
        self.test_cfg = test_cfg
        self.sections = None
        self.tests = []
        super().__init__(log, test_type, test_cfg)

    def load_test_configs(self):
        self.cfg = ConfigLoader(self.log, self.test_cfg.config_folder,
                                self.test_cfg.config_file)
        for test in self.cfg.export_tests:
            if test.export_enabled:
                self.sections = GimpExportTestSections(test, self.cfg.config_path,
                                                       self.test_cfg.data_folder,
                                                       self.log)
            else:
                pass

    def get_test_group_config(self, test_name):
        #FIXME maybe have this sorted and use a search...
        if self.sections is not None:
            for test in self.sections.test_groups:
                if test.group_name == test_name:
                    return test
        return None

    def add_test(self, test_class):
        self.log.info(f"Adding tests for {test_class.group_name}")
        self.tests.append(test_class)

    def run_tests(self):
        self.print_header(self.test_type)
        for test in self.tests:
            self.log.message(f"Running tests for {test.group_name}")
            test_config = self.get_test_group_config(test.group_name)
            if test_config is None:
                self.log.error(f"Could not find configuration for {test.group_name} tests!")
            else:
                test.execute(test_config, self.test_cfg.data_folder)
        self.print_footer("Export tests finished")

    def show_results(self):
        #FIXME export results are not implemented yet
        pass



#!/usr/bin/env python3

# NOTE: these tests were originally mostly supposed to be GUI tests, but
# the early tests were much too flimsy. And to be fair, they were not
# proper GUI tests either, as they would hack-call some core functions,
# but without actually loading the full process and clicking on buttons
# or actually interacting with the interface.
# These replacement tests are further away since they use the public API
# (but this one does use core code too!).
#
# Now the ideal implementation would use dedicated GUI testing
# frameworks, which actually simulate human interactions with GUI
# applications through automated scripts. When we'll have these, we may
# replace the below tests.

tmpdir = Gimp.temp_directory()

# Tests that the URIs are correct for a newly created image.
image = Gimp.Image.new(800, 600, Gimp.ImageBaseType.RGB)
#Gimp.Display.new(image)
gimp_assert("New image has no file associated", image.get_xcf_file() is None)
gimp_assert("New image has no imported file", image.get_imported_file() is None);
gimp_assert("New image has no exported file", image.get_exported_file() is None);
image.delete()


# Tests that GimpImage URIs are correct for an XCF file that has just been opened.
path  = os.path.join(os.environ['GIMP_TESTING_ABS_TOP_SRCDIR'],
                     'app/tests/files/gimp-2-6-file.xcf')
gimp_assert("XCF file to load exists", GLib.file_test(path, GLib.FileTest.EXISTS))
file  = Gio.File.new_for_path(path)
image = Gimp.file_load(Gimp.RunMode.NONINTERACTIVE, file)
gimp_assert("Loaded XCF's file equals the loaded file", image.get_xcf_file().equal(file));
gimp_assert("Loaded XCF has no imported file", image.get_imported_file() is None);
gimp_assert("Loaded XCF has no exported file", image.get_exported_file() is None);


# Tests that the URIs are correct for an image that has been imported and then saved.
path = os.path.join(tmpdir, 'gimp-test.xcf')
saved_file = Gio.File.new_for_path(path)
gimp_assert(f"Making sure {path} does not exists", not GLib.file_test(path, GLib.FileTest.EXISTS))
Gimp.file_save(Gimp.RunMode.NONINTERACTIVE, image, saved_file, None)
gimp_assert("XCF file was properly saved", GLib.file_test(path, GLib.FileTest.EXISTS))
gimp_assert("Saved XCF's file is correct", image.get_xcf_file().equal(saved_file));
gimp_assert("Saved XCF has no imported file", image.get_imported_file() is None);
gimp_assert("Saved XCF has no exported file", image.get_exported_file() is None);


# Tests that the URIs for an exported, newly created file are correct.
path = os.path.join(tmpdir, 'gimp-test.png')
exported_file  = Gio.File.new_for_path(path)
gimp_assert(f"Making sure {path} does not exists", not GLib.file_test(path, GLib.FileTest.EXISTS))
Gimp.file_save(Gimp.RunMode.NONINTERACTIVE, image, exported_file, None)
gimp_assert("PNG file was properly saved", GLib.file_test(path, GLib.FileTest.EXISTS))
gimp_assert("Exported image's file still equals the saved file", image.get_xcf_file().equal(saved_file));
gimp_assert("Exported image has no imported file", image.get_imported_file() is None);
gimp_assert("Exported image's exported file is correct", image.get_exported_file().equal(exported_file));

saved_file.delete()
exported_file.delete()
image.delete()


# Tests that URIs are correct for an imported image.
path  = os.path.join(os.environ['GIMP_TESTING_ABS_TOP_BUILDDIR'],
                     'gimp-data/images/logo/gimp64x64.png')
gimp_assert("Image file to import exists", GLib.file_test(path, GLib.FileTest.EXISTS))
file  = Gio.File.new_for_path(path)
image = Gimp.file_load(Gimp.RunMode.NONINTERACTIVE, file)
gimp_assert("Imported image has no file associated", image.get_xcf_file() is None)
gimp_assert("Imported image's imported file equals the loaded file", image.get_imported_file().equal(file));
gimp_assert("Imported image has no exported file", image.get_exported_file() is None);


# Tests that after a XCF file was imported then exported, the import URI is cleared.
# An image can not be considered both imported and exported at the same time.
gimp_assert(f"Making sure {exported_file.peek_path()} does not exists",
            not GLib.file_test(exported_file.peek_path(), GLib.FileTest.EXISTS))
Gimp.file_save(Gimp.RunMode.NONINTERACTIVE, image, exported_file, None)
gimp_assert("Imported then exported image has no file associated", image.get_xcf_file() is None)
gimp_assert("Imported then exported image has no imported file", image.get_imported_file() is None)
gimp_assert("Imported then exported image has a correct exported file", image.get_exported_file().equal(exported_file))

exported_file.delete()
image.delete()



#!/usr/bin/env python3

image = Gimp.Image.new(32,32,Gimp.ImageBaseType.RGB)
gimp_assert('Gimp.Image.new', image is not None)

layer1 = Gimp.Layer.new(image, "layer1", 20, 10,
                        Gimp.ImageType.RGBA_IMAGE, 100.0,
                        Gimp.LayerMode.NORMAL)
image.insert_layer(layer1,None,0)

group1 = Gimp.GroupLayer.new(image, None)
image.insert_layer(group1,None,-1)

layer2 = Gimp.Layer.new(image, "layer2", 10, 20,
                        Gimp.ImageType.RGBA_IMAGE, 100.0,
                        Gimp.LayerMode.NORMAL)

gimp_assert('insert layer inside group', image.insert_layer(layer2,group1,-1) == True)

# Make floating selection

# 1. Fail with no selection
gimp_assert('Gimp.Selection.float - no selection',
            Gimp.Selection.float(image,[layer1],10,10) is None)

# 2. Fail on a group layer
gimp_assert('Gimp.Selection.float - group layer',
            Gimp.Selection.float(image,[group1],10,10) is None)

# Create a selection
image.select_rectangle(Gimp.ChannelOps.REPLACE, 5, 5, 20, 20)

# 3. Succeed on a normal layer
gimp_assert('take selected layers: layer1',
            image.take_selected_layers([layer1]) is True)
float1 = Gimp.Selection.float(image,[layer1],10,10)
gimp_assert('Gimp.Selection.float - normal layer',
            float1 is not None)
gimp_assert('Remove float1 layer from image',
            Gimp.floating_sel_remove(float1) is True)

# Create a selection
image.select_rectangle(Gimp.ChannelOps.REPLACE, 5, 5, 20, 20)

# 4. Succeed on a layer inside a group
gimp_assert('take selected layers: layer2',
            image.take_selected_layers([layer2]) is True)
sel_drawables = image.get_selected_drawables()
gimp_assert('selected drawables', sel_drawables is not None)
float2 = Gimp.Selection.float(image,sel_drawables,10,10)
gimp_assert('Gimp.Selection.float - layer inside group', float2 is not None)
gimp_assert('Remove float2 layer from image',
            Gimp.floating_sel_remove(float2) is True)




"""
Python scripts to test GimpBrush class of libgimp API.

See comments about usage and setup in test-resource-class.py

Setup: "2. Hardness 050" brush is selected.

Testing context_set_brush
and a few other tests of class methods
are in test_resource_class.py

FUTURE: revise comparison of floats to within epsilon
"""

"""
Test a known, stock, parametric brush "2. Hardness 050"
********************************************************************************

Numeric literals might change if gimp devs change defaults.
"""
brush_default = Gimp.context_get_brush()
print( brush_default.get_id() )
assert brush_default.get_id() == "2. Hardness 050"

assert brush_default.is_generated()
assert not brush_default.is_editable()

"""
Test getters of default, stock (not editable), parametric brush
"""
success, width, height, mask_bpp, color_bpp = brush_default.get_info()
print( width, height, mask_bpp, color_bpp )
assert success
assert width     == 51
assert height    == 51
assert mask_bpp  == 1
# !!! Since parametric, not pixmap and color_bpp is zero
assert color_bpp == 0

success, width, height, mask_bpp, mask,  color_bpp, pixels = brush_default.get_pixels()
print( width, height, mask_bpp, len(mask), color_bpp, len(pixels) )
assert success
assert width     == 51
assert height    == 51
assert mask_bpp  == 1
# !!! Since parametric, not pixmap and color_bpp is zero
assert color_bpp == 0
assert len(mask)   == 2601
assert len(pixels) == 0

# !!! spacing does not return success, only a value
spacing = brush_default.get_spacing()
print(spacing)
assert spacing == 10

# shape, radius, spikes, hardness, aspect_ratio, angle
success, shape = brush_default.get_shape()
assert success
print(shape)
assert shape == Gimp.BrushGeneratedShape.CIRCLE

success, radius = brush_default.get_radius()
assert success
print(radius)
assert radius == 25.0

success, spikes = brush_default.get_spikes()
assert success
print(spikes)
assert spikes == 2

success, hardness = brush_default.get_hardness()
assert success
print(hardness)
assert hardness == 0.5

success, aspect_ratio = brush_default.get_aspect_ratio()
assert success
print(aspect_ratio)
assert aspect_ratio == 1.0

success, angle = brush_default.get_angle()
assert success
print(angle)
assert angle == 0.0


"""
Test set_spacing fails on not editable brush.
"""
success = brush_default.set_spacing(1)
assert not success
# spacing did not change
assert brush_default.get_spacing() == 10


"""
Test setters fail on non-editable brush.

When they fail, they return a value that is nullish
"""
success, returned_shape = brush_default.set_shape(Gimp.BrushGeneratedShape.DIAMOND)
print(success, returned_shape)
assert not success
assert returned_shape is Gimp.BrushGeneratedShape.CIRCLE

success, returned_radius = brush_default.set_radius(1.0)
print(success, returned_radius)
assert not success
assert returned_radius == 0.0

success, returned_spikes = brush_default.set_spikes(1.0)
print(success, returned_spikes)
assert not success
assert returned_spikes == 0

success, returned_hardness = brush_default.set_hardness(1.0)
print(success, returned_hardness)
assert not success
assert returned_hardness == 0.0

success, returned_aspect_ratio = brush_default.set_aspect_ratio(1.0)
print(success, returned_aspect_ratio)
assert not success

success, returned_angle = brush_default.set_angle(1.0)
assert not success


"""
Test non-parametric brush i.e. raster, and stock i.e. not editable
********************************************************************************

No method exists to get a specific brush.
But we can set the ID of a new brush.
"""
brush_raster = Gimp.Brush()
brush_raster.set_property("id", "Acrylic 01")
assert brush_raster.get_id()  == "Acrylic 01"
assert brush_raster.is_valid()

# raster brush is not parametric
assert not brush_raster.is_generated()

# This brush is not editable
assert not brush_raster.is_editable()

# Raster brush has spacing
spacing = brush_raster.get_spacing()
print(spacing)
assert spacing == 25

# Getters of attributes of parametric brush fail for raster brush
success, shape = brush_raster.get_shape()
assert not success
success, radius = brush_raster.get_radius()
assert not success
success, spikes = brush_raster.get_spikes()
assert not success
success, hardness = brush_raster.get_hardness()
assert not success
success, aspect_ratio = brush_raster.get_aspect_ratio()
assert not success
success, angle = brush_raster.get_angle()
assert not success


"""
Test raster brush of kind color has pixel data.
"""
brush_color = Gimp.Brush()
brush_color.set_property("id", "z Pepper")
assert brush_color.is_valid()
success, width, height, mask_bpp, mask,  color_bpp, pixels = brush_color.get_pixels()
print( width, height, mask_bpp, len(mask), color_bpp, len(pixels) )
assert success
# !!! Since is color and raster, has pixmap and color_bpp is non-zero
assert color_bpp == 3
assert len(pixels) > 0

"""
Test setting spacing on not editable, raster brush
"""
success = brush_raster.set_spacing(1)
assert not success
# spacing did not change
assert brush_raster.get_spacing() == 25


"""
Test setters fails on raster brush.
"""
success, returned_shape = brush_raster.set_shape(Gimp.BrushGeneratedShape.DIAMOND)
print(success, returned_shape)
assert not success
# Returned value is integer 0, corresponding to CIRCLE enum
assert returned_shape is Gimp.BrushGeneratedShape.CIRCLE

success, returned_radius = brush_raster.set_radius(1.0)
assert not success
success, returned_spikes = brush_raster.set_spikes(1.0)
assert not success
success, returned_hardness = brush_raster.set_hardness(1.0)
assert not success
success, returned_aspect_ratio = brush_raster.set_aspect_ratio(1.0)
assert not success
success, returned_angle = brush_raster.set_angle(1.0)
assert not success

"""
Test new brush
********************************************************************************

Parametric and editable
"""

brush_new = Gimp.Brush.new("New Brush")

# is generated and editable
assert brush_new.is_generated()
assert brush_new.is_editable()

"""
Test setters on editable brush.
"""
success = brush_new.set_spacing(20)
assert success == True
spacing = brush_new.get_spacing()
assert spacing == 20

success, returned_shape = brush_new.set_shape(Gimp.BrushGeneratedShape.DIAMOND)
print(success)
assert success == True
assert returned_shape == Gimp.BrushGeneratedShape.DIAMOND
success, shape = brush_new.get_shape()
assert success == True
assert shape == Gimp.BrushGeneratedShape.DIAMOND

success, returned_radius = brush_new.set_radius(4.0)
assert success == True
print(returned_radius)
assert returned_radius == 4.0
success, radius = brush_new.get_radius()
assert success == True
assert radius == 4.0

success, returned_spikes = brush_new.set_spikes(2)
assert success == True
assert returned_spikes == 2
success, spikes = brush_new.get_spikes()
assert success == True
assert spikes == 2

success, returned_hardness = brush_new.set_hardness(0.5)
assert success == True
print(returned_hardness)
assert returned_hardness == 0.5
success, hardness = brush_new.get_hardness()
assert success == True
assert hardness == 0.5

success, returned_aspect_ratio = brush_new.set_aspect_ratio(5.0)
assert success == True
print(returned_aspect_ratio)
assert returned_aspect_ratio == 5.0
success, aspect_ratio = brush_new.get_aspect_ratio()
assert success == True
assert aspect_ratio == 5.0

success, returned_angle = brush_new.set_angle(90.0)
assert success == True
print(returned_angle)
assert returned_angle == 90.0
success, angle = brush_new.get_angle()
assert success == True
assert angle == 90.0

"""
Test invalid enum for shape.
"""
# TODO it yields a TypeError

"""
Test clamping at upper limits.
"""

'''
Omitting this test for now.  Possible bug.
The symptoms are that it takes a long time,
and then GIMP crashes.

success, returned_radius = brush_new.set_radius(40000)
assert success == True
# upper clamp
assert returned_radius == 32767.0
success, radius = brush_new.get_radius()
assert success == True
assert radius == 32767.0
'''

success, returned_spikes = brush_new.set_spikes(22)
assert success == True
assert returned_spikes == 20
success, spikes = brush_new.get_spikes()
assert success == True
assert spikes == 20

success, returned_hardness = brush_new.set_hardness(2.0)
assert success == True
assert returned_hardness == 1.0
success, hardness = brush_new.get_hardness()
assert success == True
assert hardness == 1.0

success, returned_aspect_ratio = brush_new.set_aspect_ratio(2000)
assert success == True
assert returned_aspect_ratio == 1000.0
success, aspect_ratio = brush_new.get_aspect_ratio()
assert success == True
assert aspect_ratio == 1000.0

success, returned_angle = brush_new.set_angle(270)
assert success == True
assert returned_angle == 90
success, angle = brush_new.get_angle()
assert success == True
assert angle == 90



"""
Cleanup
"""
success = brush_new.delete()
assert success == True




"""
Python to test GimpPattern class of libgimp API.

See comments about usage and setup in test-resource-class.py
Paste into the Python Console and expect no exceptions.

Setup: "Pine" Pattern is selected.

Testing some class methods is in test_resource_class.py

Class has no setters.
Object is not editable.
Class has no is_editable() method.
"""

"""
Test a known, stock pattern "Pine"
!!! Test numeric literals must change if the pattern is changed.
"""
pattern_pine  = Gimp.context_get_pattern()
assert pattern_pine.get_id() == "Pine"

"""
Test getters
"""
success, width, height, bpp = pattern_pine.get_info()
print( width, height, bpp)
assert success
assert width  == 64
assert height == 56
assert bpp    == 3

success, width, height, bpp, pixels = pattern_pine.get_pixels()
print( len(pixels) )
assert success
assert width  == 64
assert height == 56
assert bpp    == 3
assert len(pixels) == 10752

# Not testing the pixels content



#!/usr/bin/env python3
import os
import subprocess
from pathlib import Path

os.chdir(Path(os.getenv("MESON_SOURCE_ROOT",".")))
result = subprocess.run(['git', 'status', '--untracked-files=no', '--short', '--ignore-submodules=dirty', 'gimp-data'], capture_output=True, text=True, check=True)
exit(len(result.stdout.splitlines()))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#    This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
gi.require_version('Gimp', '3.0')
from gi.repository import Gimp
gi.require_version('GimpUi', '3.0')
from gi.repository import GimpUi
gi.require_version('Gegl', '0.4')
from gi.repository import Gegl
from gi.repository import GObject
from gi.repository import GLib
from gi.repository import Gio

import os
import sys

def N_(message): return message
def _(message): return GLib.dgettext(None, message)

class Goat (Gimp.PlugIn):
    ## GimpPlugIn virtual methods ##
    def do_query_procedures(self):
        return [ "plug-in-goat-exercise-python" ]

    def do_create_procedure(self, name):
        procedure = Gimp.ImageProcedure.new(self, name,
                                            Gimp.PDBProcType.PLUGIN,
                                            self.run, None)

        procedure.set_image_types("*")
        procedure.set_sensitivity_mask (Gimp.ProcedureSensitivityMask.DRAWABLE)

        procedure.set_menu_label(_("Plug-In Example in _Python 3"))
        procedure.set_icon_name(GimpUi.ICON_GEGL)
        procedure.add_menu_path('<Image>/Filters/Development/Plug-In Examples/')

        procedure.set_documentation(_("Plug-in example in Python 3"),
                                    _("Plug-in example in Python 3"),
                                    name)
        procedure.set_attribution("Jehan", "Jehan", "2019")

        return procedure

    def run(self, procedure, run_mode, image, drawables, config, run_data):
        if len(drawables) != 1:
            msg = _("Procedure '{}' only works with one drawable.").format(procedure.get_name())
            error = GLib.Error.new_literal(Gimp.PlugIn.error_quark(), msg, 0)
            return procedure.new_return_values(Gimp.PDBStatusType.CALLING_ERROR, error)
        else:
            drawable = drawables[0]

        if run_mode == Gimp.RunMode.INTERACTIVE:
            gi.require_version('Gtk', '3.0')
            from gi.repository import Gtk
            gi.require_version('Gdk', '3.0')
            from gi.repository import Gdk

            GimpUi.init("goat-exercise-py3.py")

            dialog = GimpUi.Dialog(use_header_bar=True,
                                   title=_("Plug-In Example in Python 3"),
                                   role="goat-exercise-Python3")

            dialog.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
            dialog.add_button(_("_Source"), Gtk.ResponseType.APPLY)
            dialog.add_button(_("_OK"), Gtk.ResponseType.OK)

            geometry = Gdk.Geometry()
            geometry.min_aspect = 0.5
            geometry.max_aspect = 1.0
            dialog.set_geometry_hints(None, geometry, Gdk.WindowHints.ASPECT)

            box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=2)
            dialog.get_content_area().add(box)
            box.show()

            # XXX We use printf-style string for sharing the localized
            # string. You may just use recommended Python format() or
            # any style you like in your plug-ins.
            head_text=_("This plug-in is an exercise in '%s' to "
                        "demo plug-in creation.\nCheck out the last "
                        "version of the source code online by clicking "
                        "the \"Source\" button.") % ("Python 3")
            label = Gtk.Label(label=head_text)
            box.pack_start(label, False, False, 1)
            label.show()

            contents = None
            # Get the file contents Python-style instead of using
            # GLib.file_get_contents() which returns bytes result, and
            # when converting to string, get newlines as text contents.
            # Rather than wasting time to figure this out, use Python
            # core API!
            with open(os.path.realpath(__file__), 'r') as f:
                contents = f.read()

            if contents is not None:
                scrolled = Gtk.ScrolledWindow()
                scrolled.set_vexpand (True)
                box.pack_start(scrolled, True, True, 1)
                scrolled.show()

                view = Gtk.TextView()
                view.set_wrap_mode(Gtk.WrapMode.WORD)
                view.set_editable(False)
                buffer = view.get_buffer()
                buffer.set_text(contents, -1)
                scrolled.add(view)
                view.show()

            while (True):
                response = dialog.run()
                if response == Gtk.ResponseType.OK:
                    dialog.destroy()
                    break
                elif response == Gtk.ResponseType.APPLY:
                    url = "https://gitlab.gnome.org/GNOME/gimp/-/blob/master/extensions/goat-exercises/goat-exercise-py3.py"
                    Gio.app_info_launch_default_for_uri(url, None)
                    continue
                else:
                    dialog.destroy()
                    return procedure.new_return_values(Gimp.PDBStatusType.CANCEL,
                                                       GLib.Error())

        intersect, x, y, width, height = drawable.mask_intersect()
        if intersect:
            Gegl.init(None)

            buffer = drawable.get_buffer()
            shadow_buffer = drawable.get_shadow_buffer()

            graph = Gegl.Node()
            input = graph.create_child("gegl:buffer-source")
            input.set_property("buffer", buffer)
            invert = graph.create_child("gegl:invert")
            output = graph.create_child("gegl:write-buffer")
            output.set_property("buffer", shadow_buffer)
            input.link(invert)
            invert.link(output)
            output.process()

            # This is extremely important in bindings, since we don't
            # unref buffers. If we don't explicitly flush a buffer, we
            # may left hanging forever. This step is usually done
            # during an unref().
            shadow_buffer.flush()

            drawable.merge_shadow(True)
            drawable.update(x, y, width, height)
            Gimp.displays_flush()

        return procedure.new_return_values(Gimp.PDBStatusType.SUCCESS, GLib.Error())

Gimp.main(Goat.__gtype__, sys.argv)



#!/usr/bin/env python3

N_DEFAULT_USER_UNITS=6

gimp_unit_defs = [
  # pseudo unit
  {
    'factor':       0.0,
    'digits':       0,
    'name':         "pixels",
    'symbol':       "px",
    'abbreviation': "px"
  },
  # standard units
  {
    'factor':       1.0,
    'digits':       2,
    'name':         "inches",
    'symbol':       "''",
    'abbreviation': "in"
  },
  {
    'factor':       25.4,
    'digits':       1,
    'name':         "millimeters",
    'symbol':       "mm",
    'abbreviation': "mm"
  },
  # professional units
  {
    'factor':       72.0,
    'digits':       0,
    'name':         "points",
    'symbol':       "pt",
    'abbreviation': "pt"
  },
  {
    'factor':       6.0,
    'digits':       1,
    'name':         "picas",
    'symbol':       "pc",
    'abbreviation': "pc"
  }
]

unit = Gimp.Unit.inch()
gimp_assert('Gimp.Unit.inch()', type(unit) == Gimp.Unit)

unit2 = Gimp.Unit.inch()
gimp_assert('Gimp.Unit.inch() always returns an unique object', unit == unit2)

for i in range(len(gimp_unit_defs)):
  unit = Gimp.Unit.get_by_id(i)
  unitdef = gimp_unit_defs[i]
  gimp_assert('Testing built-in unit {}'.format(i),
              type(unit) == Gimp.Unit                            and \
              unit.get_name() == unitdef['name']                 and \
              unit.get_symbol() == unitdef['symbol']             and \
              unit.get_abbreviation() == unitdef['abbreviation'] and \
              unit.get_factor() == unitdef['factor']             and \
              unit.get_digits() == unitdef['digits'])

  if i == Gimp.UnitID.INCH:
    gimp_assert('Gimp.Unit.inch() is the same as Gimp.Unit.get_by_id (Gimp.UnitID.INCH)',
                unit == unit2)

unit = Gimp.Unit.get_by_id(Gimp.UnitID.END)
n_user_units = 0
while unit is not None:
  n_user_units += 1
  unit = Gimp.Unit.get_by_id(Gimp.UnitID.END + n_user_units)
gimp_assert('Counting default user units', n_user_units == N_DEFAULT_USER_UNITS)

unit2 = Gimp.Unit.new ("name", 2.0, 1, "symbol", "abbreviation");
gimp_assert('Gimp.Unit.new()', type(unit2) == Gimp.Unit)

gimp_assert("Verifying the new user unit's ID", unit2.get_id() == Gimp.UnitID.END + n_user_units)
gimp_assert("Verifying the new user unit's unicity", unit2 == Gimp.Unit.get_by_id(Gimp.UnitID.END + n_user_units))

unit = Gimp.Unit.get_by_id(Gimp.UnitID.END)
n_user_units = 0
while unit is not None:
  n_user_units += 1
  unit = Gimp.Unit.get_by_id(Gimp.UnitID.END + n_user_units)
gimp_assert('Counting again user units', n_user_units == N_DEFAULT_USER_UNITS + 1)



#!/usr/bin/env python3

NEW_IMAGE_WIDTH=1920
NEW_IMAGE_HEIGHT=2001

image = Gimp.Image.new(NEW_IMAGE_WIDTH, NEW_IMAGE_HEIGHT, Gimp.ImageBaseType.RGB)
gimp_assert('Gimp.Image.new()', type(image) == Gimp.Image)

images = Gimp.get_images()
gimp_assert('Gimp.get_images()', len(images) == 1 and images[0] == image)

text_layer = Gimp.TextLayer.new(image, "hello world", Gimp.context_get_font(), 20, Gimp.Unit.point())
gimp_assert('Gimp.TextLayer.new() with point unit', type(text_layer) == Gimp.TextLayer)

gimp_assert('Gimp.Image.InsertLayer()', image.insert_layer(text_layer, None, 0))

text_layer = Gimp.TextLayer.new(image, "hello world", Gimp.context_get_font(), 20, Gimp.Unit.pixel())
gimp_assert('Gimp.TextLayer.new() with pixel unit', type(text_layer) == Gimp.TextLayer)
gimp_assert('Gimp.Image.InsertLayer()', image.insert_layer(text_layer, None, 0))



#!/usr/bin/env python3

"""
performance-log-expand.py -- Delta-decodes GIMP performance logs
Copyright (C) 2018  Ell

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Usage: performance-log-expand.py < infile > outfile
"""

from xml.etree import ElementTree
import sys

empty_element = ElementTree.Element ("")

# Read performance log from STDIN
log = ElementTree.fromstring (sys.stdin.buffer.read ())

try:
    has_backtrace = bool (int (log.find ("params").find ("backtrace").text))
except:
    has_backtrace = False

def expand_simple (element, last_values):
    last_values.update ({value.tag: value.text for value in element})

    for value in list (element):
        element.remove (value)

    for tag, text in last_values.items ():
        value = ElementTree.SubElement (element, tag)
        value.text = text
        value.tail = "\n"

# Expand samples
last_vars      = {}
last_backtrace = {}

for sample in (log.find ("samples") if log.find ("samples") is not None else empty_element).iterfind ("sample"):
    # Expand variables
    vars = sample.find ("vars")
    if vars is None:
        vars = ElementTree.SubElement (sample, "vars")

    expand_simple (vars, last_vars)

    # Expand backtrace
    if has_backtrace:
        backtrace = sample.find ("backtrace")
        if backtrace is None:
            backtrace = ElementTree.SubElement (sample, "backtrace")

        for thread in backtrace:
            id     = thread.get ("id")
            head   = thread.get ("head")
            tail   = thread.get ("tail")
            attrib = dict (thread.attrib)
            frames = list (thread)

            last_thread = last_backtrace.setdefault (id, [{}, []])
            last_frames = last_thread[1]

            if head:
                frames = last_frames[:int (head)] + frames

                del attrib["head"]

            if tail:
                frames = frames + last_frames[-int (tail):]

                del attrib["tail"]

            last_thread[0] = attrib
            last_thread[1] = frames

            if not frames and thread.text is None:
                del last_backtrace[id]

        for thread in list (backtrace):
            backtrace.remove (thread)

        for id, (attrib, frames) in last_backtrace.items ():
            thread = ElementTree.SubElement (backtrace, "thread", attrib)
            thread.text = "\n"
            thread.tail = "\n"

            thread.extend (frames)

# Expand address map
last_address = {}

for address in (log.find ("address-map") if log.find ("address-map") is not None else empty_element).iterfind ("address"):
    expand_simple (address, last_address)

# Write performance log to STDOUT
sys.stdout.buffer.write (ElementTree.tostring (log))
